<?xml version="1.0" encoding="UTF-8"?>

<!--
Copyright (c) Vanderbilt University, 2000-2002
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.

File: 					UdmToXMI.XSLT
Author: 					Endre Magyari <endre.magyari@vanderbilt.edu>
Synopsis:				Converts a UDM UML metamodel to metainformation in XMI 1.0 format based on UML 1.3

CHANGELOG:
	11/28/03  -	minor bugfix with isNavigable attribute values
	11/30/02	-	error when generating generalizations, it did not worked that way with Xalan	
	11/24/02	-	fixed datatypes. Dut to a  Copy&Paste error,  the UDM Boolean data type was missing
	11/24/02	-	fixed typo when evaluating nonpersistent attribute of attribute, it was "nonpersitent"
	11/19/02	-	fixed some ordering problems in case of classes and assocclasses.
	11/19/02	-	first complete version. The output conforms to UML.DTD distributed by the OMG.
					all the UML features supported by UDM are supported and translated.
	
	-->

<xsl:stylesheet version='1.0' xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
	<xsl:output  method='xml' version='1.0' encoding='UTF-8'  doctype-system='uml.dtd'   indent='yes'/>
	
	<!-- Random generated UUIID prefix-->	
	<xsl:variable name='uuid_prefix'>-63--26--54-118-9b87f6:f1647ded57</xsl:variable>
	
	<!-- _id is a key in the document to be transformed -->
	<xsl:key name='id' match='Class' use='@_id'/>


	
	<!-- All the XMI headers to be generated when the root element is matched -->
	<xsl:template match='/'>
		<XMI xmi.version='1.0'>
			<XMI.header>
	   			<XMI.documentation>
	   				<XMI.exporter>Mobies XMI exporter</XMI.exporter>
	   				<XMI.exporterVersion>1.30</XMI.exporterVersion>
	   			</XMI.documentation>
	   			<XMI.metamodel xmi.name='UML' xmi.version='1.3'/>
	 	 	</XMI.header>
	 		 <XMI.content>
				<xsl:apply-templates/>
			</XMI.content>
		</XMI>
	</xsl:template>

	<!-- Attribute -->
	<xsl:template match='Attribute'>
		<Foundation.Core.Attribute>

			<!-- We use generate-id() here. It will never be referenced-->
			<xsl:attribute name='xmi.id'>
				<xsl:value-of select='generate-id()'/>
			</xsl:attribute>
			<xsl:attribute name='xmi.uuid'>
				<xsl:value-of select='$uuid_prefix'/>:<xsl:value-of select='generate-id()'/>
			</xsl:attribute>

			<Foundation.Core.ModelElement.name><xsl:value-of select='@name'/></Foundation.Core.ModelElement.name>
			<Foundation.Core.ModelElement.visibility xmi.value='{@visibility}'/>
			<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
			<Foundation.Core.Feature.ownerScope xmi.value='instance'/>
			<Foundation.Core.StructuralFeature.multiplicity>
				<Foundation.Data_Types.Multiplicity xmi.id="{concat('FDTM_',generate-id())}">
					<Foundation.Data_Types.Multiplicity.range>
						<Foundation.Data_Types.MultiplicityRange xmi.id="{concat('FDTMR_',generate-id())}">
							<Foundation.Data_Types.MultiplicityRange.lower><xsl:value-of select='@min'/></Foundation.Data_Types.MultiplicityRange.lower>
							<Foundation.Data_Types.MultiplicityRange.upper><xsl:value-of select='@max'/></Foundation.Data_Types.MultiplicityRange.upper>
						</Foundation.Data_Types.MultiplicityRange>
					</Foundation.Data_Types.Multiplicity.range>
				</Foundation.Data_Types.Multiplicity>
			</Foundation.Core.StructuralFeature.multiplicity>
			<Foundation.Core.StructuralFeature.changeability xmi.value='changeable'/>
			<xsl:choose>
				<xsl:when test='@defvalue'>
					<Foundation.Core.Attribute.initialValue>
						<Foundation.Data_Types.Expression xmi.id="{concat('FDTE_',generate-id())}">
							<Foundation.Data_Types.Expression.language>Java</Foundation.Data_Types.Expression.language>
							<Foundation.Data_Types.Expression.body><xsl:value-of select='@defvalue'/></Foundation.Data_Types.Expression.body>
						</Foundation.Data_Types.Expression>
		              	</Foundation.Core.Attribute.initialValue>
	              	</xsl:when>
              	</xsl:choose>

              	<Foundation.Core.Feature.owner>
              		<Foundation.Core.Classifier xmi.idref="{concat('xmi.',substring-after(../@_id,'id'))}"/>
              	</Foundation.Core.Feature.owner>
              	<Foundation.Core.StructuralFeature.type>
             			<Foundation.Core.Classifier xmi.idref="{concat('xmi.dt_',@type)}"/>
              	</Foundation.Core.StructuralFeature.type>
              	<Foundation.Core.ModelElement.taggedValue>
				<Foundation.Extension_Mechanisms.TaggedValue xmi.id="{concat('FEMTVT_',generate-id())}">
					<Foundation.Extension_Mechanisms.TaggedValue.tag>transient</Foundation.Extension_Mechanisms.TaggedValue.tag>
		              	<Foundation.Extension_Mechanisms.TaggedValue.value>false</Foundation.Extension_Mechanisms.TaggedValue.value>
		              	<Foundation.Extension_Mechanisms.TaggedValue.modelElement>
		              		<Foundation.Core.ModelElement xmi.idref="{generate-id()}"/>
		              	</Foundation.Extension_Mechanisms.TaggedValue.modelElement>
				</Foundation.Extension_Mechanisms.TaggedValue>
	             <Foundation.Extension_Mechanisms.TaggedValue xmi.id="{concat('FEMTVV_',generate-id())}">
				<Foundation.Extension_Mechanisms.TaggedValue.tag>volatile</Foundation.Extension_Mechanisms.TaggedValue.tag>
	                  	<Foundation.Extension_Mechanisms.TaggedValue.value><xsl:value-of select='@nonpersistent'/></Foundation.Extension_Mechanisms.TaggedValue.value>
	                  	<Foundation.Extension_Mechanisms.TaggedValue.modelElement>
	                  		<Foundation.Core.ModelElement xmi.idref='{generate-id()}'/>
	                  	</Foundation.Extension_Mechanisms.TaggedValue.modelElement>
				</Foundation.Extension_Mechanisms.TaggedValue>
			</Foundation.Core.ModelElement.taggedValue>
		</Foundation.Core.Attribute>
	</xsl:template>

	<!-- AssociationRole -->
	<xsl:template match='AssociationRole'>
		<Foundation.Core.AssociationEnd>
			
			<!-- We use generate-id() here. It will never be referenced-->
			<xsl:attribute name='xmi.id'>
				<xsl:value-of select='generate-id()'/>
			</xsl:attribute>
			<xsl:attribute name='xmi.uuid'>
				<xsl:value-of select='$uuid_prefix'/>:<xsl:value-of select='generate-id()'/>
			</xsl:attribute>

			<Foundation.Core.ModelElement.name>
				<xsl:value-of select='@name'/>
			</Foundation.Core.ModelElement.name>
			
			<Foundation.Core.ModelElement.visibility xmi.value='public'/>
			<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
			<Foundation.Core.AssociationEnd.isNavigable>
	              	<xsl:attribute name='xmi.value'>
		              	<xsl:choose>
						<xsl:when test='@isNavigable'>
							<xsl:value-of select='@isNavigable'/>
						</xsl:when>
						<xsl:otherwise>	
							<xsl:text>true</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
	              </Foundation.Core.AssociationEnd.isNavigable>
	             <Foundation.Core.AssociationEnd.ordering xmi.value='unordered'/>
			<Foundation.Core.AssociationEnd.aggregation xmi.value='none'/>
			<Foundation.Core.AssociationEnd.targetScope xmi.value='instance'/>
			<Foundation.Core.AssociationEnd.multiplicity>
				<Foundation.Data_Types.Multiplicity xmi.id="{concat('FDTM_',generate-id())}">
					<Foundation.Data_Types.Multiplicity.range>
						<Foundation.Data_Types.MultiplicityRange xmi.id="{concat('FDTMR_', generate-id())}">
							<Foundation.Data_Types.MultiplicityRange.lower><xsl:value-of select='@min'/></Foundation.Data_Types.MultiplicityRange.lower>
							<Foundation.Data_Types.MultiplicityRange.upper><xsl:value-of select='@max'/></Foundation.Data_Types.MultiplicityRange.upper>
						</Foundation.Data_Types.MultiplicityRange>
					</Foundation.Data_Types.Multiplicity.range>
				</Foundation.Data_Types.Multiplicity>
			</Foundation.Core.AssociationEnd.multiplicity>
			<Foundation.Core.AssociationEnd.changeability xmi.value='changeable'/>
			<Foundation.Core.AssociationEnd.association>
				<Foundation.Core.Association xmi.idref='{generate-id(..)}'>
					<!-- if parent has attribute @assocClass than we use it's @assocClass as idref
						if not, generate-id(..)-->
					<xsl:attribute name='xmi.idref'>
						<xsl:choose>
							<xsl:when test='../@assocClass'>
								<xsl:value-of select="concat('xmi.',substring-after(../@assocClass,'id'))"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select='generate-id()'/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
				</Foundation.Core.Association>
			</Foundation.Core.AssociationEnd.association>
			<Foundation.Core.AssociationEnd.type>
				<!-- this points to a class -->
				<Foundation.Core.Classifier xmi.idref="{concat('xmi.',substring-after(@target,'id'))}"/>
			</Foundation.Core.AssociationEnd.type>
		</Foundation.Core.AssociationEnd>
	</xsl:template>




	<!-- CompositionChildRole -->
	<xsl:template match='CompositionChildRole'>
		<Foundation.Core.AssociationEnd>
			
			<!-- We use generate-id() here. It will never be referenced-->
			<xsl:attribute name='xmi.id'>
				<xsl:value-of select='generate-id()'/>
			</xsl:attribute>
			<xsl:attribute name='xmi.uuid'>
				<xsl:value-of select='$uuid_prefix'/>:<xsl:value-of select='generate-id()'/>
			</xsl:attribute>

			<Foundation.Core.ModelElement.name>
				<xsl:value-of select='@name'/>
			</Foundation.Core.ModelElement.name>
			
			<Foundation.Core.ModelElement.visibility xmi.value='public'/>
			<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
			<Foundation.Core.AssociationEnd.isNavigable>
	              	<xsl:attribute name='xmi.value'>
		              	<xsl:choose>
						<xsl:when test='@isNavigable'>
							<xsl:value-of select='@isNavigable'/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>true</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
	              </Foundation.Core.AssociationEnd.isNavigable>
	             <Foundation.Core.AssociationEnd.ordering xmi.value='unordered'/>
			<Foundation.Core.AssociationEnd.aggregation xmi.value='none'/>
			<Foundation.Core.AssociationEnd.targetScope xmi.value='instance'/>
			<Foundation.Core.AssociationEnd.multiplicity>
				<Foundation.Data_Types.Multiplicity xmi.id="{concat('FDTM_',generate-id())}">
					<Foundation.Data_Types.Multiplicity.range>
						<Foundation.Data_Types.MultiplicityRange xmi.id="{concat('FDTMR_', generate-id())}">
							<Foundation.Data_Types.MultiplicityRange.lower><xsl:value-of select='@min'/></Foundation.Data_Types.MultiplicityRange.lower>
							<Foundation.Data_Types.MultiplicityRange.upper><xsl:value-of select='@max'/></Foundation.Data_Types.MultiplicityRange.upper>
						</Foundation.Data_Types.MultiplicityRange>
					</Foundation.Data_Types.Multiplicity.range>
				</Foundation.Data_Types.Multiplicity>
			</Foundation.Core.AssociationEnd.multiplicity>
			<Foundation.Core.AssociationEnd.changeability xmi.value='changeable'/>
			<Foundation.Core.AssociationEnd.association>
				<Foundation.Core.Association xmi.idref='{generate-id(..)}'/>
			</Foundation.Core.AssociationEnd.association>
			<Foundation.Core.AssociationEnd.type>
				<!-- this points to a class -->
				<Foundation.Core.Classifier xmi.idref="{concat('xmi.',substring-after(@target,'id'))}"/>
			</Foundation.Core.AssociationEnd.type>
		</Foundation.Core.AssociationEnd>
	</xsl:template>

	<!-- CompositionParentRole -->
	<!-- this translates int an XMI AssociationEnd with aggregation = 'composite', 
		multiplicity=[1,1], classifier = target class, and association = parent node -->
	<xsl:template match='CompositionParentRole'>
		<Foundation.Core.AssociationEnd>		
			<!-- We use generate-id() here. It will never be referenced-->
			<xsl:attribute name='xmi.id'>
				<xsl:value-of select='generate-id()'/>
			</xsl:attribute>
			<xsl:attribute name='xmi.uuid'>
				<xsl:value-of select='$uuid_prefix'/>:<xsl:value-of select='generate-id()'/>
			</xsl:attribute>
			
	              <Foundation.Core.ModelElement.name>
	              	<xsl:value-of select='@name'/>
	              </Foundation.Core.ModelElement.name>
	              
	              <Foundation.Core.ModelElement.visibility xmi.value='public'/>
	              <Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
	              <Foundation.Core.AssociationEnd.isNavigable>
	              	<xsl:attribute name='xmi.value'>
		              	<xsl:choose>
						<xsl:when test='@isNavigable'>
							<xsl:value-of select='@isNavigable'/>
						</xsl:when>
						<xsl:otherwise>	
							<xsl:text>true</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
	              </Foundation.Core.AssociationEnd.isNavigable>
	              <Foundation.Core.AssociationEnd.ordering xmi.value='unordered'/>
	              <Foundation.Core.AssociationEnd.aggregation xmi.value='composite'/>
	              <Foundation.Core.AssociationEnd.targetScope xmi.value='instance'/>
	              <Foundation.Core.AssociationEnd.multiplicity>
	              	<Foundation.Data_Types.Multiplicity xmi.id="{concat('FDTM_', generate-id())}">
					<Foundation.Data_Types.Multiplicity.range>
						<Foundation.Data_Types.MultiplicityRange xmi.id="{concat('FDTMR_',generate-id())}">
							<Foundation.Data_Types.MultiplicityRange.lower>	1</Foundation.Data_Types.MultiplicityRange.lower>
		                      		<Foundation.Data_Types.MultiplicityRange.upper>	1</Foundation.Data_Types.MultiplicityRange.upper>
		                   		 </Foundation.Data_Types.MultiplicityRange>
		                   	</Foundation.Data_Types.Multiplicity.range>
		             </Foundation.Data_Types.Multiplicity>
	              </Foundation.Core.AssociationEnd.multiplicity>
	              
			<Foundation.Core.AssociationEnd.changeability xmi.value='changeable'/>
			<Foundation.Core.AssociationEnd.association>
				<Foundation.Core.Association xmi.idref='{generate-id(..)}'/>
	              </Foundation.Core.AssociationEnd.association>
			<Foundation.Core.AssociationEnd.type>
				<!-- this is a class -->
				<Foundation.Core.Classifier xmi.idref="{concat('xmi.',substring-after(@target,'id'))}"/>
			</Foundation.Core.AssociationEnd.type>
		</Foundation.Core.AssociationEnd>			
	</xsl:template>
	
	<!-- Association -->
	<xsl:template match='Association'>
		<xsl:choose>
			<xsl:when test='@assocClass'>
				<Foundation.Core.AssociationClass>		
					<!--Since this is also a class, we may use the assocClass attribute value as id -->
					<xsl:attribute name='xmi.id'>
						<xsl:value-of select="concat('xmi.',substring-after(@assocClass,'id'))"/>
					</xsl:attribute>
					<xsl:attribute name='xmi.uuid'>
						<xsl:value-of select='$uuid_prefix'/>:<xsl:value-of select="substring-after(@assocClass,'id')"/>
					</xsl:attribute>

					<!-- This the name of the association class pointed by assocClass and not the name of the association ! -->
					<Foundation.Core.ModelElement.name>
						<xsl:for-each select="key('id',@assocClass)">
							<!-- We rely on the fact that there is only one such attribute -->
							<xsl:value-of select='@name'/>
						</xsl:for-each>
					</Foundation.Core.ModelElement.name>
					<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isRoot xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isLeaf xmi.value='false'/>					
					<Foundation.Core.GeneralizableElement.isAbstract>
						<xsl:attribute name='xmi.value'>
							<xsl:for-each select="key('id',@assocClass)">
								<!-- We rely on the fact that there is only one such attribute -->
								<xsl:value-of select='@isAbstract'/>
							</xsl:for-each>
						</xsl:attribute>
					</Foundation.Core.GeneralizableElement.isAbstract>
				
					<!-- Namespace -->
					<Foundation.Core.ModelElement.namespace>
						<Foundation.Core.Namespace xmi.idref='{generate-id(..)}'/>
		          		</Foundation.Core.ModelElement.namespace>
		          	
					<!-- Inheritence -->
					<xsl:for-each select="key('id',@assocClass)">	
						<!-- Inheritence: Generalization(Basetypes) -->
	          				<xsl:choose>
	          					<xsl:when test='@baseTypes'>
	          						<Foundation.Core.GeneralizableElement.generalization>
		          						<xsl:call-template name='gen_gen_references'>
										<xsl:with-param name='str' select='normalize-space(@baseTypes)'/>
										<xsl:with-param name='base' select='@_id'/>
										<xsl:with-param name='direction' select='false()'/>
									</xsl:call-template>
								</Foundation.Core.GeneralizableElement.generalization>
	          					</xsl:when>
	          				</xsl:choose>
						<!-- Inheritence: Specialization(Subtypes) -->
	          				<xsl:choose>
	          					<xsl:when test='@subTypes'>
	          						<Foundation.Core.GeneralizableElement.specialization>
		          						<xsl:call-template name='gen_gen_references'>
										<xsl:with-param name='str' select='normalize-space(@subTypes)'/>
										<xsl:with-param name='base' select='@_id'/>
										<xsl:with-param name='direction' select='true()'/>
									</xsl:call-template>
								</Foundation.Core.GeneralizableElement.specialization>
	          					</xsl:when>
	          				</xsl:choose>
					</xsl:for-each>
			
					<!-- AssociationRoles -->
					<Foundation.Core.Association.connection>
		          			<!-- go recursive, search for AssociationRoles -->
		          			<xsl:apply-templates/>
					</Foundation.Core.Association.connection>
					
					
					<xsl:for-each select="key('id',@assocClass)">
						<!-- Attributes -->
						<xsl:choose>
							<xsl:when test='child::*'>
								<!-- The class has attributes -->
								<Foundation.Core.Classifier.feature>
									<!-- search for Attributes -->
									<xsl:apply-templates/>
		          					</Foundation.Core.Classifier.feature>
          						</xsl:when>
          					</xsl:choose>
					</xsl:for-each>
		          	</Foundation.Core.AssociationClass>
	          	</xsl:when>
	          	<xsl:otherwise>
	          		<Foundation.Core.Association>		
					<!--we use generate ID here.  -->
					<xsl:attribute name='xmi.id'>
						<xsl:value-of select='generate-id()'/>
					</xsl:attribute>
					<xsl:attribute name='xmi.uuid'>
						<xsl:value-of select='$uuid_prefix'/>:<xsl:value-of select='generate-id()'/>
					</xsl:attribute>
					
					<Foundation.Core.ModelElement.name>
						<xsl:value-of select='@name'/>
					</Foundation.Core.ModelElement.name>
					<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isRoot xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isLeaf xmi.value='false'/>					
					<Foundation.Core.GeneralizableElement.isAbstract xmi.value='false'/>	
					<Foundation.Core.ModelElement.namespace>
						<Foundation.Core.Namespace xmi.idref='{generate-id(..)}'/>
		          		</Foundation.Core.ModelElement.namespace>
		          		<Foundation.Core.Association.connection>
		          			<!-- go recursive, search for AssociationRoles -->
		          			<xsl:apply-templates/>
					</Foundation.Core.Association.connection>
		          	</Foundation.Core.Association>
	          	</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
		

	<!-- Composition -->
	<xsl:template match='Composition'>
		<Foundation.Core.Association>	
			<!-- We use generate-id() here. It will never be referenced-->
			<xsl:attribute name='xmi.id'>
				<xsl:value-of select='generate-id()'/>
			</xsl:attribute>
			<xsl:attribute name='xmi.uuid'>
				<xsl:value-of select='$uuid_prefix'/>:<xsl:value-of select='generate-id()'/>
			</xsl:attribute>
		
			<Foundation.Core.ModelElement.name>
				<xsl:value-of select='@name'/>
			</Foundation.Core.ModelElement.name>
			<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
			<Foundation.Core.GeneralizableElement.isRoot xmi.value='false'/>
			<Foundation.Core.GeneralizableElement.isLeaf xmi.value='false'/>					
			<Foundation.Core.GeneralizableElement.isAbstract xmi.value='false'/>	
			<Foundation.Core.ModelElement.namespace>
				<Foundation.Core.Namespace xmi.idref='{generate-id(..)}'/>
          		</Foundation.Core.ModelElement.namespace>
          		<Foundation.Core.Association.connection>
          			<!-- go recursive -->
          			<xsl:apply-templates/>
			</Foundation.Core.Association.connection>
          	</Foundation.Core.Association>
	</xsl:template>

	<!-- Class -->		
	<xsl:template match='Class'>
		<!-- we should skip the declaration if the class is an association class,
			because it gets generated as an Fundation.Core.AssociationClass
			 upon the <Association> elements -->
		<xsl:choose>
			<xsl:when test='not(@association)'>
				<Foundation.Core.Class>
					<!-- The attribute id needs to be computed -->
					<xsl:attribute name='xmi.id'>
						<xsl:choose>
							<xsl:when test='@_id'>
								<xsl:value-of select="concat('xmi.',substring-after(@_id,'id'))"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:message terminate='yes'>
									Class <xsl:value-of select='@name'/> does not have a mandatory id attribute!
								</xsl:message>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:attribute>
				
					<!-- The attribute uuid will be prefix + generated-id -->			
					<xsl:attribute name='xmi.uuid'>
						<xsl:value-of select='$uuid_prefix'/>:<xsl:value-of select='generate-id()'/>
					</xsl:attribute>	
					
					<Foundation.Core.ModelElement.name>
						<xsl:value-of select='@name'/>
					</Foundation.Core.ModelElement.name>
					<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isRoot xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isLeaf xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isAbstract xmi.value='{@isAbstract}'/>
					<Foundation.Core.Class.isActive xmi.value='false'/>
					<Foundation.Core.ModelElement.namespace>
						<Foundation.Core.Namespace xmi.idref='{generate-id(..)}'/>
					</Foundation.Core.ModelElement.namespace>
					<!-- Inheritence: Generalization(Basetypes) -->
          				<xsl:choose>
          					<xsl:when test='@baseTypes'>
          						<Foundation.Core.GeneralizableElement.generalization>
	          						<xsl:call-template name='gen_gen_references'>
									<xsl:with-param name='str' select='normalize-space(@baseTypes)'/>
									<xsl:with-param name='base' select='@_id'/>
									<xsl:with-param name='direction' select='false()'/>
								</xsl:call-template>
							</Foundation.Core.GeneralizableElement.generalization>
          					</xsl:when>
          				</xsl:choose>
					<!-- Inheritence: Specialization(Subtypes) -->
          				<xsl:choose>
          					<xsl:when test='@subTypes'>
          						<Foundation.Core.GeneralizableElement.specialization>
	          						<xsl:call-template name='gen_gen_references'>
									<xsl:with-param name='str' select='normalize-space(@subTypes)'/>
									<xsl:with-param name='base' select='@_id'/>
									<xsl:with-param name='direction' select='true()'/>
								</xsl:call-template>
							</Foundation.Core.GeneralizableElement.specialization>
          					</xsl:when>
          				</xsl:choose>
					
					<!-- Attributes -->
					<xsl:choose>
						<xsl:when test='child::*'>
							<Foundation.Core.Classifier.feature>
								<xsl:apply-templates/>
		          				</Foundation.Core.Classifier.feature>
          					</xsl:when>
          				</xsl:choose>
		        	</Foundation.Core.Class>
		        </xsl:when>
		        <xsl:otherwise/><!-- Class is an Association Class. It will declared as such -->
        	</xsl:choose>	 
		
   	</xsl:template>

	<!-- recursive template to generate the references to generalization elements in class elements,
	in case of generalizations and specializations -->
	<xsl:template name='gen_gen_references'>
		<xsl:param name='str'/>
		<xsl:param name='base'/>
		<xsl:param name='direction'/>
		<xsl:if test='string-length($str) > 0'>
			<!-- computing the derived id -->
			<xsl:variable name='derived'>
				<xsl:choose>
					<xsl:when test="contains($str, ' ')">
						<xsl:value-of select="substring-before($str, ' ')"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select='$str'/>
					</xsl:otherwise>
				</xsl:choose> 
			</xsl:variable>
			<!-- generate some output-->
			<Foundation.Core.Generalization>
				<xsl:attribute name='xmi.idref'>
					<xsl:choose>
						<xsl:when test='$direction = true()'>
							<xsl:value-of select="concat('xmi.gen_', concat($base, $derived))"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="concat('xmi.gen_', concat($derived, $base))"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
			</Foundation.Core.Generalization>
			
			<!-- recursive step -->
			<xsl:call-template name='gen_gen_references'>
				<xsl:with-param name='str' select="normalize-space(substring-after($str, ' '))"/>
				<xsl:with-param name='base' select='$base'/>
				<xsl:with-param name='direction' select='$direction'/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	
	<!-- recursive template to generate generalization elements from string idref list -->
	<xsl:template name='tokenize'>
		<xsl:param name='str'/>
		<xsl:param name='base'/>		
		<xsl:if test='string-length($str) > 0'>
			<Foundation.Core.Generalization>
				
				<!-- computing the derived id -->
				<xsl:variable name='derived'>
						<xsl:choose>
							<xsl:when test="contains($str, ' ')">
								<xsl:value-of select="substring-before($str, ' ')"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select='$str'/>
							</xsl:otherwise>
						</xsl:choose> 
				</xsl:variable>
				
				<!-- this generalization element must be referenced in the Class element either as specialization or as generalization,
				so we will generate some IDs which can be easily computed while matching the class elements as well. -->
				<xsl:attribute name='xmi.id'>
					<xsl:value-of select="concat('xmi.gen_', concat($base, $derived))"/>
				</xsl:attribute>
				
				<xsl:attribute name='xmi.uuid'>
					<xsl:value-of select="concat($uuid_prefix, concat($base, $derived))"/>
				</xsl:attribute>
				
				<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
				<Foundation.Core.ModelElement.namespace>
					<!-- Namespace is the generate-id() for the diagram, which is the parent of the base class-->
					<xsl:for-each select="key('id', $base)">
						<Foundation.Core.Namespace xmi.idref='{generate-id(..)}'/>
					</xsl:for-each>
				</Foundation.Core.ModelElement.namespace>				
				<Foundation.Core.Generalization.child>
					<Foundation.Core.GeneralizableElement xmi.idref="{concat('xmi.',substring-after($derived,'id'))}"/>
				</Foundation.Core.Generalization.child>
				<Foundation.Core.Generalization.parent>
					<Foundation.Core.GeneralizableElement xmi.idref="{concat('xmi.',substring-after($base,'id'))}"/>
	          		</Foundation.Core.Generalization.parent>
			</Foundation.Core.Generalization>
			<xsl:call-template name='tokenize'>
				<xsl:with-param name='str' select="normalize-space(substring-after($str, ' '))"/>
				<xsl:with-param name='base' select='$base'/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	
	<!-- Inheritence -->	
	<xsl:template name='generalizations'>
	
		<!-- TODO! -->
		<xsl:for-each select='Class'>
			<xsl:variable name='base' select='@_id'/>
			<xsl:for-each select='@subTypes'>
					<xsl:call-template name='tokenize'>
					<xsl:with-param name='str' select="normalize-space(.)"/>
					<xsl:with-param name='base' select='$base'/>
				</xsl:call-template>
			</xsl:for-each>
		</xsl:for-each>
	</xsl:template>	
	
	
	
	<!-- The root of the input XML is always a Diagram-->
	<xsl:template match='Diagram'>
		<Model_Management.Model>
		
			<!-- We use generate-id() here. It will be referenced from all classes-->
			<xsl:attribute name='xmi.id'>
				<xsl:value-of select='generate-id()'/>
			</xsl:attribute>
			<xsl:attribute name='xmi.uuid'>
				<xsl:value-of select='$uuid_prefix'/>:<xsl:value-of select='generate-id()'/>
			</xsl:attribute>	
			
			<Foundation.Core.ModelElement.name>
				<xsl:value-of select='@name'/>
			</Foundation.Core.ModelElement.name>				
			
			<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
		      <Foundation.Core.GeneralizableElement.isRoot xmi.value='false'/>
		      <Foundation.Core.GeneralizableElement.isLeaf xmi.value='false'/>
		      <Foundation.Core.GeneralizableElement.isAbstract xmi.value='false'/>
		      <Foundation.Core.Namespace.ownedElement>
		      <!-- Go recursive -->
				<xsl:apply-templates/>
			<!-- Add inheritence relationships -->
			<xsl:call-template name='generalizations'/>
			
			<!-- Define datatypes -->
				<!--Integer -->
				 <Foundation.Core.DataType xmi.id='xmi.dt_Integer'>
					<Foundation.Core.ModelElement.name>Integer</Foundation.Core.ModelElement.name>
					<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isRoot xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isLeaf xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isAbstract xmi.value='false'/>
					<Foundation.Core.ModelElement.namespace>
						<Foundation.Core.Namespace xmi.idref='{generate-id()}'/>
					</Foundation.Core.ModelElement.namespace>
	       		</Foundation.Core.DataType>
	       		<!--String -->
				 <Foundation.Core.DataType xmi.id='xmi.dt_String'>
					<Foundation.Core.ModelElement.name>String</Foundation.Core.ModelElement.name>
					<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isRoot xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isLeaf xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isAbstract xmi.value='false'/>
					<Foundation.Core.ModelElement.namespace>
						<Foundation.Core.Namespace xmi.idref='{generate-id()}'/>
					</Foundation.Core.ModelElement.namespace>
	       		</Foundation.Core.DataType>
	       		<!--Boolean -->
				 <Foundation.Core.DataType xmi.id='xmi.dt_Boolean'>
					<Foundation.Core.ModelElement.name>Boolean</Foundation.Core.ModelElement.name>
					<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isRoot xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isLeaf xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isAbstract xmi.value='false'/>
					<Foundation.Core.ModelElement.namespace>
						<Foundation.Core.Namespace xmi.idref='{generate-id()}'/>
					</Foundation.Core.ModelElement.namespace>
	       		</Foundation.Core.DataType>
	       		<!--Real -->
				 <Foundation.Core.DataType xmi.id='xmi.dt_Real'>
					<Foundation.Core.ModelElement.name>Real</Foundation.Core.ModelElement.name>
					<Foundation.Core.ModelElement.isSpecification xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isRoot xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isLeaf xmi.value='false'/>
					<Foundation.Core.GeneralizableElement.isAbstract xmi.value='false'/>
					<Foundation.Core.ModelElement.namespace>
						<Foundation.Core.Namespace xmi.idref='{generate-id()}'/>
					</Foundation.Core.ModelElement.namespace>
	       		</Foundation.Core.DataType>	
			</Foundation.Core.Namespace.ownedElement>
		</Model_Management.Model>
	</xsl:template>
</xsl:stylesheet>