<?xml version="1.0" encoding="UTF-8"?>
<!--%%
Copyright (c) Vanderbilt University, 2000-2002
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
%
File: 					XMI2Udm.XSLT
Author: 					Endre Magyari <endre.magyari@vanderbilt.edu>
Synopsis:				Converts an XMI 1.0/UML 1.3 format metainformation to UDM UML metamodell
CHANGELOG:
%	
	28/11/03	-	Changed to produce XSD based output instead of DTD
	05/19/03	-	Minor Bug: Composition shoud have it's subelements ordered
	11/25/02	-	first complete version. The output conforms to UML.DTD distributed by the MoBIES/UDM
					all the UML features supported by UDM are supported and translated.
%
%-->
<xsl:stylesheet version="1.0" 
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" />
	<xsl:key name="id" match="node()" use="@xmi.id"/>
	<xsl:namespace-alias stylesheet-prefix="xmlns" result-prefix="xsi"></xsl:namespace-alias>
	<!-- xmi.id is a key in the document to be transformed -->
	<xsl:template name="Foundation.Core.Classifier.feature">
		<!-- Classifier Features -->
		<xsl:apply-templates select="Foundation.Core.Attribute"/>
	</xsl:template>
	<xsl:template match="Foundation.Core.Attribute">
		<!-- Attributes -->
		<Attribute name="{Foundation.Core.ModelElement.name}" min="{Foundation.Core.StructuralFeature.multiplicity/Foundation.Data_Types.Multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.lower}" max="{Foundation.Core.StructuralFeature.multiplicity/Foundation.Data_Types.Multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.upper}" visibility="{Foundation.Core.ModelElement.visibility/@xmi.value}" ordered="false">
			<!-- nonpersistent is a taggedvalue named volatile -->
			<xsl:for-each select="Foundation.Core.ModelElement.taggedValue/Foundation.Extension_Mechanisms.TaggedValue">
				<xsl:choose>
					<xsl:when test="string(Foundation.Extension_Mechanisms.TaggedValue.tag) = 'volatile'">
						<xsl:attribute name="nonpersistent"><xsl:value-of select="Foundation.Extension_Mechanisms.TaggedValue.value"/></xsl:attribute>
					</xsl:when>
				</xsl:choose>
			</xsl:for-each>
			<!-- defval is Foundation.Core.Attribute.initial -->
			<xsl:for-each select="Foundation.Core.Attribute.initialValue/Foundation.Data_Types.Expression">
				<xsl:attribute name="defvalue"><xsl:value-of select="Foundation.Data_Types.Expression.body"/></xsl:attribute>
			</xsl:for-each>
			<!-- type is obtained via a key to a datatype -->
			<xsl:for-each select="key('id', Foundation.Core.StructuralFeature.type/Foundation.Core.Classifier/@xmi.idref)">
				<xsl:attribute name="type"><xsl:value-of select="Foundation.Core.ModelElement.name"/></xsl:attribute>
			</xsl:for-each>
		</Attribute>
	</xsl:template>
	<!-- Namespace.ownedElement; do nothing, go recursive -->
	<xsl:template match="Foundation.Core.Namespace.ownedElement">
		<!-- First, the association definitions, without association classes 
			(This is part of the Association definitions generation 	-->
		<xsl:for-each select="Foundation.Core.Association">
			<xsl:variable name="assoc_pos" select="number(position())"/>
			<xsl:choose>
				<xsl:when test="count(Foundation.Core.Association.connection/Foundation.Core.AssociationEnd/Foundation.Core.AssociationEnd.aggregation[@xmi.value = 'composite']) = 0">
					<Association _id="{concat('id', string(number($assoc_pos) + number($assoc_pos)*1000 + 400000))}">
						<!-- Check if name exists -->
						<xsl:choose>
							<xsl:when test="string-length(Foundation.Core.ModelElement.name) > 1">
								<xsl:attribute name="name"><xsl:value-of select="Foundation.Core.ModelElement.name"/></xsl:attribute>
							</xsl:when>
						</xsl:choose>
						<xsl:for-each select="Foundation.Core.Association.connection/Foundation.Core.AssociationEnd">
							<AssociationRole _id="{concat('id',string(number($assoc_pos)*1000 + 500000 + position()))}" min="{string(Foundation.Core.AssociationEnd.multiplicity/Foundation.Data_Types.Multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.lower)}" max="{string(Foundation.Core.AssociationEnd.multiplicity/Foundation.Data_Types.Multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.upper)}" target="{concat('id',substring-after(Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref,'xmi.'))}">
								<!-- Check if name exists -->
								<xsl:choose>
									<xsl:when test="string-length(Foundation.Core.ModelElement.name) > 1">
										<xsl:attribute name="name"><xsl:value-of select="Foundation.Core.ModelElement.name"/></xsl:attribute>
									</xsl:when>
								</xsl:choose>
							</AssociationRole>
						</xsl:for-each>
					</Association>
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
		<!-- The  the association definitions, with association class 
			(this is still part of the Association definition generation -->
		<xsl:for-each select="Foundation.Core.AssociationClass">
			<xsl:variable name="assoc_pos" select="position() + count(../Foundation.Core.Association)"/>
			<xsl:choose>
				<xsl:when test="count(Foundation.Core.Association.connection/Foundation.Core.AssociationEnd/Foundation.Core.AssociationEnd.aggregation[@xmi.value = 'composite']) = 0">
					<Association _id="{concat('id', string(number($assoc_pos) + number($assoc_pos)*1000 + 400000))}" assocClass="{concat('id', substring-after(@xmi.id,'xmi.'))}">
						<!-- Check if name exists -->
						<xsl:choose>
							<xsl:when test="string-length(Foundation.Core.ModelElement.name) > 1">
								<xsl:attribute name="name"><xsl:value-of select="Foundation.Core.ModelElement.name"/></xsl:attribute>
							</xsl:when>
						</xsl:choose>
						<xsl:for-each select="Foundation.Core.Association.connection/Foundation.Core.AssociationEnd">
							<AssociationRole _id="{concat('id',500000 +$assoc_pos*1000 +  position())}" min="{string(Foundation.Core.AssociationEnd.multiplicity/Foundation.Data_Types.Multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.lower)}" max="{string(Foundation.Core.AssociationEnd.multiplicity/Foundation.Data_Types.Multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.upper)}" target="{concat('id',substring-after(Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref,'xmi.'))}">
								<!-- Check if name exists -->
								<xsl:choose>
									<xsl:when test="string-length(Foundation.Core.ModelElement.name) > 1">
										<xsl:attribute name="name"><xsl:value-of select="Foundation.Core.ModelElement.name"/></xsl:attribute>
									</xsl:when>
								</xsl:choose>
							</AssociationRole>
						</xsl:for-each>
					</Association>
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
		<!-- go through classes that are not association classes 
			(Class generation) 		-->
		<xsl:for-each select="Foundation.Core.Class">
			<Class _id="{concat('id',substring-after(@xmi.id, 'xmi.'))}" name="{string(Foundation.Core.ModelElement.name)}" isAbstract="{Foundation.Core.GeneralizableElement.isAbstract/@xmi.value}">
				<xsl:variable name="this_class_id" select="@xmi.id"/>
				<!-- computing child roles -->
				<xsl:variable name="childRoles">
					<xsl:for-each select="../Foundation.Core.Association">
						<xsl:variable name="assoc_pos" select="number(position())"/>
						<xsl:for-each select="Foundation.Core.Association.connection">
							<!-- it will have two association ends -->
							<xsl:choose>
								<!-- need to check that's it a composition. This is done by checking the number of association ends that have the aggregation attribute set to 'composite' -->
								<xsl:when test="count(Foundation.Core.AssociationEnd/Foundation.Core.AssociationEnd.aggregation[@xmi.value = 'composite']) = 1">
									<xsl:for-each select="Foundation.Core.AssociationEnd">
										<xsl:choose>
											<!-- AssociationEnd point to this class -->
											<xsl:when test="string(Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref) = string($this_class_id)">
												<xsl:choose>
													<!-- get the child part of it -->
													<xsl:when test="string(Foundation.Core.AssociationEnd.aggregation/@xmi.value) = 'none'">
														<xsl:variable name="child_role_id" select="position() + number($assoc_pos)*1000 + 100000"/>
														<xsl:value-of select="concat(concat('id',$child_role_id), ' ')"/>
													</xsl:when>
												</xsl:choose>
											</xsl:when>
										</xsl:choose>
									</xsl:for-each>
								</xsl:when>
							</xsl:choose>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<!-- end of child roles computation -->
				<!-- computing parent roles -->
				<xsl:variable name="parentRoles">
					<xsl:for-each select="../Foundation.Core.Association">
						<xsl:variable name="assoc_pos" select="number(position())"/>
						<xsl:for-each select="Foundation.Core.Association.connection">
							<!-- it will have two association ends -->
							<xsl:choose>
								<!-- need to check that's it a composition. This is done by checking the number of association ends that have the aggregation attribute set to 'composite' -->
								<xsl:when test="count(Foundation.Core.AssociationEnd/Foundation.Core.AssociationEnd.aggregation[@xmi.value = 'composite']) = 1">
									<xsl:for-each select="Foundation.Core.AssociationEnd">
										<xsl:choose>
											<!-- AssociationEnd point to this class -->
											<xsl:when test="string(Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref) = string($this_class_id)">
												<xsl:choose>
													<!-- get the child part of it -->
													<xsl:when test="string(Foundation.Core.AssociationEnd.aggregation/@xmi.value) = 'composite'">
														<xsl:variable name="child_role_id" select="position() + number($assoc_pos)*1000 + 200000"/>
														<xsl:value-of select="concat(concat('id',$child_role_id), ' ')"/>
													</xsl:when>
												</xsl:choose>
											</xsl:when>
										</xsl:choose>
									</xsl:for-each>
								</xsl:when>
							</xsl:choose>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<!-- end of child roles computation -->
				<!-- computing basetypes and subtypes -->
				<xsl:variable name="baseTypes">
					<xsl:for-each select="Foundation.Core.GeneralizableElement.generalization/Foundation.Core.Generalization">
						<xsl:for-each select="key('id',@xmi.idref)">
							<xsl:for-each select="Foundation.Core.Generalization.parent/Foundation.Core.GeneralizableElement">
								<!-- it will be only one -->
								<xsl:value-of select="concat(concat('id', substring-after(@xmi.idref,'xmi.')),' ')"/>
							</xsl:for-each>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<xsl:variable name="subTypes">
					<xsl:for-each select="Foundation.Core.GeneralizableElement.specialization/Foundation.Core.Generalization">
						<xsl:for-each select="key('id',@xmi.idref)">
							<xsl:for-each select="Foundation.Core.Generalization.child/Foundation.Core.GeneralizableElement">
								<!-- it will be only one -->
								<xsl:value-of select="concat(concat('id', substring-after(@xmi.idref,'xmi.')),' ')"/>
							</xsl:for-each>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<!-- writing out childroles, if any -->
				<xsl:choose>
					<xsl:when test="string-length($childRoles) >1">
						<xsl:attribute name="childRoles"><xsl:value-of select="$childRoles"/></xsl:attribute>
					</xsl:when>
				</xsl:choose>
				<!-- writing out parentroles, if any -->
				<xsl:choose>
					<xsl:when test="string-length($parentRoles) > 1">
						<xsl:attribute name="parentRoles"><xsl:value-of select="$parentRoles"/></xsl:attribute>
					</xsl:when>
				</xsl:choose>
				<!-- writing out baseTypes, if any -->
				<xsl:choose>
					<xsl:when test="string-length($baseTypes) > 1">
						<xsl:attribute name="baseTypes"><xsl:value-of select="$baseTypes"/></xsl:attribute>
					</xsl:when>
				</xsl:choose>
				<!-- writing out subTypes, if any -->
				<xsl:choose>
					<xsl:when test="string-length($subTypes) > 1">
						<xsl:attribute name="subTypes"><xsl:value-of select="$subTypes"/></xsl:attribute>
					</xsl:when>
				</xsl:choose>
				<!-- Go recursive -->
				<xsl:apply-templates select="Foundation.Core.Classifier.feature"/>
			</Class>
		</xsl:for-each>
		<!-- Go through association classes,	(Class generation)		-->
		<xsl:for-each select="Foundation.Core.AssociationClass">
			<xsl:variable name="assoc_poss" select="position() + count(../Foundation.Core.Association)"/>
			<Class _id="{concat('id',substring-after(@xmi.id, 'xmi.'))}" name="{string(Foundation.Core.ModelElement.name)}" isAbstract="{Foundation.Core.GeneralizableElement.isAbstract/@xmi.value}" association="{concat('id', string(number($assoc_poss) + number($assoc_poss)*1000 + 400000))}">
				<xsl:variable name="this_class_id" select="@xmi.id"/>
				<!-- computing child roles -->
				<xsl:variable name="childRoles">
					<xsl:for-each select="../Foundation.Core.Association">
						<xsl:variable name="assoc_pos" select="position()"/>
						<xsl:for-each select="Foundation.Core.Association.connection">
							<!-- it will have two association ends -->
							<xsl:choose>
								<!-- need to check that's it a composition. This is done by checking the number of association ends that have the aggregation attribute set to 'composite' -->
								<xsl:when test="count(Foundation.Core.AssociationEnd/Foundation.Core.AssociationEnd.aggregation[@xmi.value = 'composite']) = 1">
									<xsl:for-each select="Foundation.Core.AssociationEnd">
										<xsl:choose>
											<!-- AssociationEnd point to this class -->
											<xsl:when test="string(Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref) = string($this_class_id)">
												<xsl:choose>
													<!-- get the child part of it -->
													<xsl:when test="string(Foundation.Core.AssociationEnd.aggregation/@xmi.value) = 'none'">
														<xsl:variable name="child_role_id" select="position() + number($assoc_pos)*1000 + 100000"/>
														<xsl:value-of select="concat(concat('id',$child_role_id), ' ')"/>
													</xsl:when>
												</xsl:choose>
											</xsl:when>
										</xsl:choose>
									</xsl:for-each>
								</xsl:when>
							</xsl:choose>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<!-- end of child roles computation -->
				<!-- computing parent roles -->
				<xsl:variable name="parentRoles">
					<xsl:for-each select="../Foundation.Core.Association">
						<xsl:variable name="assoc_pos" select="position()"/>
						<xsl:for-each select="Foundation.Core.Association.connection">
							<!-- it will have two association ends -->
							<xsl:choose>
								<!-- need to check that's it a composition. This is done by checking the number of association ends that have the aggregation attribute set to 'composite' -->
								<xsl:when test="count(Foundation.Core.AssociationEnd/Foundation.Core.AssociationEnd.aggregation[@xmi.value = 'composite']) = 1">
									<xsl:for-each select="Foundation.Core.AssociationEnd">
										<xsl:choose>
											<!-- AssociationEnd point to this class -->
											<xsl:when test="string(Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref) = string($this_class_id)">
												<xsl:choose>
													<!-- get the child part of it -->
													<xsl:when test="string(Foundation.Core.AssociationEnd.aggregation/@xmi.value) = 'composite'">
														<xsl:variable name="child_role_id" select="position() + number($assoc_pos)*1000 + 200000"/>
														<xsl:value-of select="concat(concat('id',$child_role_id), ' ')"/>
													</xsl:when>
												</xsl:choose>
											</xsl:when>
										</xsl:choose>
									</xsl:for-each>
								</xsl:when>
							</xsl:choose>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<!-- end of child roles computation -->
				<!-- computing basetypes and subtypes -->
				<xsl:variable name="baseTypes">
					<xsl:for-each select="Foundation.Core.GeneralizableElement.generalization/Foundation.Core.Generalization">
						<xsl:for-each select="key('id',@xmi.idref)">
							<xsl:for-each select="Foundation.Core.Generalization.parent/Foundation.Core.GeneralizableElement">
								<!-- it will be only one -->
								<xsl:value-of select="concat(concat('id', substring-after(@xmi.idref,'xmi.')),' ')"/>
							</xsl:for-each>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<xsl:variable name="subTypes">
					<xsl:for-each select="Foundation.Core.GeneralizableElement.specialization/Foundation.Core.Generalization">
						<xsl:for-each select="key('id',@xmi.idref)">
							<xsl:for-each select="Foundation.Core.Generalization.child/Foundation.Core.GeneralizableElement">
								<!-- it will be only one -->
								<xsl:value-of select="concat(concat('id', substring-after(@xmi.idref,'xmi.')),' ')"/>
							</xsl:for-each>
						</xsl:for-each>
					</xsl:for-each>
				</xsl:variable>
				<!-- writing out childroles, if any -->
				<xsl:choose>
					<xsl:when test="string-length($childRoles) > 1">
						<xsl:attribute name="childRoles"><xsl:value-of select="$childRoles"/></xsl:attribute>
					</xsl:when>
				</xsl:choose>
				<!-- writing out parentroles, if any -->
				<xsl:choose>
					<xsl:when test="string-length($parentRoles) > 1">
						<xsl:attribute name="parentRoles"><xsl:value-of select="$parentRoles"/></xsl:attribute>
					</xsl:when>
				</xsl:choose>
				<!-- writing out baseTypes, if any -->
				<xsl:choose>
					<xsl:when test="string-length($baseTypes) > 1">
						<xsl:attribute name="baseTypes"><xsl:value-of select="$baseTypes"/></xsl:attribute>
					</xsl:when>
				</xsl:choose>
				<!-- writing out subTypes, if any -->
				<xsl:choose>
					<xsl:when test="string-length($subTypes) > 1">
						<xsl:attribute name="subTypes"><xsl:value-of select="$subTypes"/></xsl:attribute>
					</xsl:when>
				</xsl:choose>
				<!-- Go recursive -->
				<xsl:apply-templates select="Foundation.Core.Classifier.feature"/>
			</Class>
		</xsl:for-each>
		<!-- Composition defintions -->
		<xsl:for-each select="Foundation.Core.Association">
			<xsl:variable name="assoc_pos" select="position()"/>
			<xsl:choose>
				<xsl:when test="count(Foundation.Core.Association.connection/Foundation.Core.AssociationEnd/Foundation.Core.AssociationEnd.aggregation[@xmi.value = 'composite']) = 1">
					<Composition>
						<!-- Check if name exists -->
						<xsl:choose>
							<xsl:when test="string-length(Foundation.Core.ModelElement.name) > 1">
								<xsl:attribute name="name"><xsl:value-of select="Foundation.Core.ModelElement.name"/></xsl:attribute>
							</xsl:when>
						</xsl:choose>
						<xsl:for-each select="Foundation.Core.Association.connection/Foundation.Core.AssociationEnd">
							<xsl:choose>
								<xsl:when test="Foundation.Core.AssociationEnd.aggregation/@xmi.value = 'composite'">
									<!-- UDM's UML.DTD requests elements in the alphabetical order of their type-->
									<xsl:for-each select="../Foundation.Core.AssociationEnd">
										<xsl:choose>
											<xsl:when test="Foundation.Core.AssociationEnd.aggregation/@xmi.value != 'composite'">
												<CompositionChildRole _id="{concat('id',string(number($assoc_pos)*1000 + 100000 + position()))}" min="{Foundation.Core.AssociationEnd.multiplicity/Foundation.Data_Types.Multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.lower}" max="{Foundation.Core.AssociationEnd.multiplicity/Foundation.Data_Types.Multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.upper}" target="{concat('id',substring-after(Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref,'xmi.'))}">
													<!-- Check if name exists -->
													<xsl:choose>
														<xsl:when test="string-length(Foundation.Core.ModelElement.name) > 1">
															<xsl:attribute name="name"><xsl:value-of select="Foundation.Core.ModelElement.name"/></xsl:attribute>
														</xsl:when>
													</xsl:choose>
												</CompositionChildRole>
											</xsl:when>
										</xsl:choose>
									</xsl:for-each>
									<CompositionParentRole _id="{concat('id',string(number($assoc_pos)*number('1000') + number('200000') + number(position())))}" target="{concat('id',substring-after(Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref,'xmi.'))}">
										<!-- Check if name exists -->
										<xsl:choose>
											<xsl:when test="string-length(Foundation.Core.ModelElement.name) > 1">
												<xsl:attribute name="name"><xsl:value-of select="Foundation.Core.ModelElement.name"/></xsl:attribute>
											</xsl:when>
										</xsl:choose>
									</CompositionParentRole>
								</xsl:when>
							</xsl:choose>
						</xsl:for-each>
					</Composition>
				</xsl:when>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	<!-- match these and do nothing -->
	<xsl:template match="Foundation.Core.ModelElement.name"/>
	<xsl:template match="Foundation.Core.ModelElement.isSpecification"/>
	<xsl:template match="Foundation.Core.GeneralizableElement.isRoot"/>
	<xsl:template match="Foundation.Core.GeneralizableElement.isLeaf"/>
	<xsl:template match="Foundation.Core.GeneralizableElement.isAbstract"/>
	<!-- Match Model_Management.Model ; generate diagram and go recursive-->
	<xsl:template match="Model_Management.Model">
		<Diagram name="{string(Foundation.Core.ModelElement.name)}">
			<xsl:attribute name="xsi:noNamespaceSchemaLocation">Uml.xsd</xsl:attribute>				
			<xsl:apply-templates/>
		</Diagram>
	</xsl:template>
	<xsl:template match="XMI.header"/>
	<!-- XMI header; do nothing -->
	<xsl:template match="XMI.content">
		<!-- XMI.content; do nothing & go recursive -->
		<xsl:apply-templates/>
	</xsl:template>
	<xsl:template match="/">
		<!-- Root ; do nothing & go recursive -->
		<xsl:apply-templates/>
	</xsl:template>
</xsl:stylesheet>
