/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

/*
CHANGELOG:

	12/20/04	-	endre

		took the functions from UdmCppH.cpp
		Also separated the huge generator function into a bunch of smaller functions to increase readability

*/

#include "Uml.h"
#include "UmlExt.h"
#include <time.h>
#include <map>
string GetTime();
using namespace Uml;
typedef vector<Uml::Class> UmlClasses;



/*
	Declarations in the namespace
*/
void GenerateCPPDeclareMetaClasses(const Uml::Diagram & dgr, ostream & output)
{
	//Uml::Class A::meta;
	set<Uml::Class> classes = dgr.classes();
	set<Uml::Class>::iterator c;
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		output << "\tUml::Class " << cl.name() << "::meta;" << endl;
	}
};
void GenerateCPPDeclareAttributes(const Uml::Diagram & dgr, ostream & output)
{
	set<Uml::Class> classes = dgr.classes();
	set<Uml::Class>::iterator c;
	//	Uml::Attribute A::meta_a;
	for(c = classes.begin(); c != classes.end(); c++ )	
	{
		Uml::Class cl = *c;
		//Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));
		set<Uml::Attribute> attributes = cl.attributes();
		for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
		{
			output << "\tUml::Attribute " << cl.name() << "::meta_" << (*i).name() << ";" << endl;
		}
	}
};
void GenerateCPPDeclareAssociationRoles(const Uml::Diagram &dgr, const Uml::Diagram &cross_dgr, ostream & output)
{
//declarations for associationroles
	//	Uml::AssociationRole A::meta_dst;
	set<Uml::Class> classes = dgr.classes();
	set<Uml::Class>::iterator c;
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
	
		set<Uml::AssociationRole> assocs = cl.associationRoles();
		for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
		{
			Uml::Association pp = (*i).parent();
			Uml::Class aclass = pp.assocClass();
			string aname(MakeRoleName(Uml::theOther(*i)));
			if(!aclass) 
			{
				output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ";" << endl;
			}
			else 
			{
				output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ", " << cl.name() << "::meta_" << aname << "_rev;" << endl;
			}
		}

		/*
			Here is printed the decalaration of the cross-association roles
		*/
		if (cross_dgr &&(cross_dgr != dgr) )
		{
			//--//Uml::Class cross_cl = Uml::classByName(cross_dgr, cl.name());
			string cross_cl_name = string(cl.name()) + string(Udm::cross_delimiter) + string(dgr.name());
			Uml::Class cross_cl = Uml::classByName(cross_dgr, cross_cl_name );
			if(cross_cl)
			{
				set<Uml::AssociationRole> assocs = cross_cl.associationRoles();
				for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
				{
					Uml::Association pp = (*i).parent();
					Uml::Class aclass = pp.assocClass();
					string aname(MakeRoleName(Uml::theOther(*i)));
					if(!aclass) 
					{
						output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ";" << endl;
					}
					else 
					{
						output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << ", " << cl.name() << "::meta_" << aname << "_rev;" << endl;
					}
				}
			};
		};
		Uml::Association assoc = cl.association();
		string s = assoc.meta.name();
		if(assoc) 
		{
			set<Uml::AssociationRole> assocs = assoc.roles();
			for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
			{
				string aname(MakeRoleName(*i));
				string oname(((Uml::Class)(*i).target()).name());
				if(aname.size()) 
				{
				  output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << "_end_;" << endl;
				}
			}
		};
		if (cross_dgr &&(cross_dgr != dgr) )
		{
			//--//Uml::Class cross_cl = Uml::classByName(cross_dgr, cl.name());
			Uml::Class cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(dgr.name())) );
			if(cross_cl)
			{
				Uml::Association assoc = cross_cl.association();
				string s = assoc.meta.name();
				if(assoc) 
				{
					set<Uml::AssociationRole> assocs = assoc.roles();
					for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
					{
						string aname(MakeRoleName(*i));
						string oname(((Uml::Class)(*i).target()).name());
						if(aname.size()) 
						{
							output << "\tUml::AssociationRole " << cl.name() << "::meta_" << aname << "_end_;" << endl;
						}
					}
				}
			}
		};
	};//end for cycle for declaration for associationRoles
};
void GenerateCPPDeclareCompositionRoles(const Uml::Diagram &dgr, ostream& output)
{
	set<Uml::Class> classes = dgr.classes();
	set<Uml::Class>::iterator c;
	
//	Uml::CompositionParentRole A::meta_parent;
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::CompositionChildRole> children = cl.childRoles();
		for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) {
			string aname(MakeRoleName(Uml::theOther(*i)));
			output << "\tUml::CompositionParentRole " << cl.name() << "::meta_" << aname << ";" << endl;
		}
	}
//	Uml::CompositionChildRole D::meta_as;
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::CompositionParentRole> children = cl.parentRoles();
		for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) {
			string aname(MakeRoleName(Uml::theOther(*i)));
			output << "\tUml::CompositionChildRole " << cl.name() << "::meta_" << aname << ";" << endl;
		}
	}
};
void GenerateCPPDeclareConstraints(const Uml::Diagram &dgr, ostream & output)
{
	set<Uml::Class> classes = dgr.classes();
	set<Uml::Class>::iterator c;
	
//  Uml::Constraint D::meta_constraint
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::Constraint> constraints = cl.constraints();
		for( set<Uml::Constraint>::iterator i = constraints.begin(); i != constraints.end(); i++) 
			output << "\tUml::Constraint " << cl.name() << "::meta_" << i->name() << ";" << endl;
	}

//	Uml::ConstraintDefinition D::meta_constraint_definition
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::ConstraintDefinition> definitions = cl.definitions();
		for( set<Uml::ConstraintDefinition>::iterator i = definitions.begin(); i != definitions.end(); i++) 
			output << "\tUml::ConstraintDefinition " << cl.name() << "::meta_" << i->name() << ";" << endl;
	}
};

/*

	Generators of functions, outside Initialize()
*/
void GenerateCPPCreatesFunction(const Uml::Diagram &dgr, ostream & output)
{
	set<Uml::Class> classes = dgr.classes();
	set<Uml::Class>::iterator c;
	
//void Create()
// {

	output << "\tvoid Creates()" << endl << "\t{" << endl;
		
//	A::meta = Uml::CreateClass();
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		output << "\t\t" << cl.name() << "::meta = Uml::Class::Create(umldiagram);" << endl;
	}

//	A::meta_a = Uml::CreateAttribute();
	for(c = classes.begin(); c != classes.end(); c++ )	
	{
		Uml::Class cl = *c;
		
		set<Uml::Attribute> attributes = cl.attributes();
		for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
		{
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << " = Uml::Attribute::Create(" << cl.name()<<"::meta);" << endl;
		}

			//constraints
		set<Uml::Constraint> constraints = cl.constraints();
		for (set<Uml::Constraint>::iterator j = constraints.begin(); j != constraints.end(); j++)
		{
			output << "\t\t" << cl.name() << "::meta_" << j->name() << " = Uml::Constraint::Create(" << cl.name() << "::meta);" << endl;
		}

			//constraintdefinitions
		set<Uml::ConstraintDefinition> definitions = cl.definitions();
		for (set<Uml::ConstraintDefinition>::iterator k = definitions.begin(); k != definitions.end(); k++)
		{
			output << "\t\t" << cl.name() << "::meta_" << k->name() << " = Uml::ConstraintDefinition::Create(" << cl.name() << "::meta);" << endl;
		}
	}
	output << "\t}" << endl;
};

void GenerateCPPInitClassesAttributes(const Uml::Diagram & diagram, const Uml::Diagram & cross_dgr, ostream & output)
{
	//void InitAttributes()
// {
	output << "\tvoid InitClassesAttributes()" << endl << "\t{" << endl;
	set<Uml::Class> classes = diagram.classes();
	set<Uml::Class>::iterator c;
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		string ster;
		if(((string)cl.stereotype()).length()) ster = string(", \"")+(string)cl.stereotype()+"\"";
		output << "\t\t" << cl.name() << "::meta.name() = \"" << cl.name() << "\";" <<endl;
		output << "\t\t" << cl.name() << "::meta.isAbstract() = " << (cl.isAbstract() ? "true" : "false") << ";" << endl;
		
		if (!ster.empty()) 
			output << "\t\t" << cl.name() << "::meta.stereotype() = \"" << cl.stereotype() << "\";" <<endl;

		string from = cl.from();
		if (!from.empty())
			output << "\t\t" << cl.name() << "::meta.from() = \"" << cl.from() << "\";" <<endl;
	}

	output << "\t\t" << endl;

//		Uml::InitAttribute(A::meta_a, A::meta, "a", "String");
	for(c = classes.begin(); c != classes.end(); c++ )	
	{
		Uml::Class cl = *c;
		set<Uml::Attribute> attributes = cl.attributes();
		for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
		{
			if (i->defvalue())
			{
				vector<string> dva = i->defvalue();
				output << "\t\tvector<string> " << cl.name() << "_" << (*i).name() << "_dva;" <<endl;
				vector<string>::const_iterator dva_ci = dva.begin();
				while (dva_ci != dva.end())
				{
					output << "\t\t" << cl.name() << "_" <<(*i).name() << "_dva.push_back(\"" << *dva_ci++ << "\");" << endl; 
				};
			}
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".name() = \"" << (*i).name() << "\";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".type() = \"" << (*i).type() << "\";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".min() = " << (*i).min() << ";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".max() = " << (*i).max() << ";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".nonpersistent() = " << ((*i).nonpersistent() ? "true" : "false") << ";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".registry() = " << ((*i).registry() ? "true" : "false") << ";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".ordered() = " << ((*i).ordered() ? "true" : "false") << ";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".visibility() = \"" << (*i).visibility() << "\";" << endl;
			if (i->defvalue())
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".defvalue() = " << cl.name() << "_" <<(*i).name() << "_dva;" << endl;
			else
				output << "\t\t" << cl.name() << "::meta_" << (*i).name() << ".defvalue() = vector<string>();" << endl;
		}

		set<Uml::Constraint> constraints= cl.constraints();
		for( set<Uml::Constraint>::iterator j = constraints.begin(); j != constraints.end(); j++) 
		{
			output << "\t\t" << cl.name() << "::meta_" << j->name() << ".name() = \"" << j->name() << "\";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << j->name() << ".description() = \"" << j->description() << "\";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << j->name() << ".expression() = \"" << j->expression() << "\";" << endl;
		}

		set<Uml::ConstraintDefinition> defs = cl.definitions();
		for( set<Uml::ConstraintDefinition>::iterator k = defs.begin(); k != defs.end(); k++) 
		{
			output << "\t\t" << cl.name() << "::meta_" << k->name() << ".name() = \"" << k->name() << "\";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << k->name() << ".stereotype() = \"" << k->stereotype() << "\";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << k->name() << ".parameterList() = \"" << k->parameterList() << "\";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << k->name() << ".expression() = \"" << k->expression() << "\";" << endl;
			output << "\t\t" << cl.name() << "::meta_" << k->name() << ".returnType() = \"" << k->returnType() << "\";" << endl;
			
		}

		if (cross_dgr &&(cross_dgr != diagram))
		{
			Uml::Class cross_cl;
			Uml::Association ass;
			if (cross_dgr)
			{
				//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
				cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name())) );
			}
			if (cross_cl)
				ass = cross_cl.association();
			if (ass)
			{
				set<Uml::AssociationRole> asrs = ass.roles();
				set<Uml::AssociationRole>::iterator asrs_i = asrs.begin();
				while (asrs_i != asrs.end())
				{
					output << " \t\t" << cl.name() << "::meta_" << asrs_i->name() << "_end_ = " 
						<< cross_dgr.name() << "::" << ((Uml::Class)((Uml::theOther(*asrs_i)).target())).name()<<"::meta_" << asrs_i->name() << ";" <<endl;
					asrs_i++;
				}
			};
		}
	}
	output << "\t\t" << endl;
	output << "\t}" << endl;
};
void GenerateCompositionInit(const Uml::Composition &c, ostream &output, int funct_no)
{
	output << "\tvoid InitComposition" << funct_no << "()" << endl << "\t{" << endl;
	output << "\t\tUml::Composition comp = Uml::Composition::Create(umldiagram);" << endl;

	if (!((string)c.name()).empty())output << "\t\t\tcomp.name() =\"" << c.name() << "\";" <<endl;
	output << "\t\tcomp.nonpersistent() =" << (c.nonpersistent() ? "true" : "false") << ";" << endl;

	Uml::CompositionParentRole x1 = c.parentRole();
	Uml::CompositionChildRole x2 = c.childRole();

	Uml::CompositionParentRole zzp = c.parentRole();
	Uml::CompositionChildRole zzc = c.childRole();
	string aname(MakeRoleName(zzp));
	output << "\t\t" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname <<" = Uml::CompositionParentRole::Create(comp);" << endl;
	output << "\t\t" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname <<".name() = \"" << aname << "\";" << endl;
	output << "\t\t" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname <<".isNavigable() = " << (zzp.isNavigable() ? "true;" : "false;") << endl;
	output << "\t\t" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname <<".target() = " << ((Uml::Class)zzp.target()).name() << "::meta;" << endl;

	string cname(MakeRoleName(zzc));
	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << " = Uml::CompositionChildRole::Create(comp);" << endl;

	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".name() = \"" << MakeShortRoleName(zzc) << "\";" << endl;
	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".min() = " << zzc.min() << ";"<< endl;
	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".max() = " << zzc.max() << ";"<< endl;
	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".isNavigable() = " << (zzc.isNavigable() ? "true;" : "false;") << endl;
	output << "\t\t" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << ".target() = " << ((Uml::Class)zzc.target()).name() << "::meta;" << endl;

	output << "\t}" << endl;
};

void GenerateAssociationInit(const Uml::Association &a,  ostream &output, int funct_no)
{
	output << "\tvoid InitAssociation" << funct_no << "()" << endl << "\t{" << endl;

	set<Uml::AssociationRole> ar = a.roles();
	ASSERT(ar.size() == 2);
	
	output << "\t\tUml::Association ass = Uml::Association::Create(umldiagram);" << endl;

	Uml::Class aclass = a.assocClass();
	if (!((string)a.name()).empty())output << "\t\tass.name() = \"" << a.name() << "\";" <<endl;
	if(aclass) 
	{
		output << "\t\tass.assocClass() = " << aclass.name() << "::meta;" << endl;
	}

	set<Uml::AssociationRole> roles = a.roles();
	for( set<Uml::AssociationRole>::iterator i = roles.begin(); i != roles.end(); i++) 
	{
		Uml::AssociationRole zz	= Uml::theOther(*i);
		string aname(MakeRoleName(zz));
		Uml::Class cl = i->target();
		
		output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::AssociationRole::Create(ass);" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".name() = \"" << aname << "\";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".min() = " << zz.min() << ";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".max() = " << zz.max() << ";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".isNavigable() = " << (zz.isNavigable() ? "true":"false") << ";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".isPrimary() = " << (zz.isPrimary() ? "true":"false") << ";" <<endl;
		output << "\t\t" << cl.name() << "::meta_" << aname << ".target() = " << ((Uml::Class)zz.target()).name() << "::meta;" <<endl;

		Uml::Class aclass = a.assocClass();
		if(aclass) 
		{
			Uml::Class cl2 = zz.target();
			output << "\t\t" << aclass.name() << "::meta_" << aname << "_end_ = " <<
							cl2.name() << "::meta_" << MakeRoleName(*i) << "_rev = " << 
							cl.name() << "::meta_" << aname << ";" << endl;
		}
	}
	
	
	output << "\t};" << endl;

}

int GenerateCPPAssociationInitFunctions(const Uml::Diagram & diagram, ostream &output)
{
	int ass_inits = 0;
	const set<Uml::Association> &asss = diagram.associations();
	set<Uml::Association>::const_iterator a;
	for(a = asss.begin(); a != asss.end(); a++ )	
		GenerateAssociationInit(*a, output, ass_inits++);
	return ass_inits;	
};
int GenerateCPPCompositionInitFunctions(const Uml::Diagram & diagram, ostream &output)
{
	int comp_inits = 0;
	const set<Uml::Composition> &comps = diagram.compositions();
	set<Uml::Composition>::const_iterator c;
	for(c = comps.begin(); c != comps.end(); c++ )	
		GenerateCompositionInit(*c, output, comp_inits++);

	return comp_inits;
}

/*
	Functions which build the Initalize() function
*/
void GenerateCPPCrossObjectInits(const Uml::Diagram &diagram, const Uml::Diagram & cross_dgr, ostream & output)
{
	set<Uml::Class> classes = diagram.classes();
	set<Uml::Class>::iterator c;
	if (cross_dgr &&(cross_dgr != diagram))
	{
		//		M2::meta_src = CL::M2::meta_src;		
		for(c = classes.begin(); c != classes.end(); c++ )	
		{
			Uml::Class cl = *c;
			Uml::Class cross_cl;
			if (cross_dgr)
			{
				cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name())) );
				//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
			}
			if (cross_cl)
			{
				set<Uml::AssociationRole> ar = cross_cl.associationRoles();
				set<Uml::AssociationRole>::iterator ar_i = ar.begin();
				while (ar_i != ar.end())
				{
					output << "\t\t" << cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
					" = " << cross_dgr.name()<< "::" << cross_cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name()<< ";" <<endl;

					Uml::AssociationRole zz	= Uml::theOther(*ar_i);
					string aname(MakeRoleName(zz));

					Uml::Association ass = ar_i->parent();
					Uml::Class a_class = ass.assocClass();
					
					if(a_class) 
					{
						Uml::Class cl2 = zz.target();
						
						output << "\t\t" << cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
						"_rev = " << cross_dgr.name()<< "::" << cl2.name() << "::meta_"<<ar_i->name()<< ";" <<endl;

					}
					ar_i++;
				}
			}
		}
	}
};
void GenerateCPPInitializeInheritence(const Uml::Diagram &dgr, ostream & output)
{
	set<Uml::Class> classes = dgr.classes();;
	set<Uml::Class>::iterator c;
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::Class> st = cl.subTypes();
		for( set<Uml::Class>::iterator i = st.begin(); i != st.end(); i++) 
			output << "\t\t" << cl.name() << "::meta.subTypes() +=  " << (*i).name() << "::meta;" << endl;
	}	
};

void GenerateCPPStaticClass(const Uml::Diagram &diagram, ostream & output)
{
	output << "\tstatic struct _regClass" << endl;
	output << "\t{" << endl;
	output << "\t\t_regClass()" << endl;
	output << "\t\t{" << endl;
	output << "\t\t\tUdm::MetaDepository::StoreDiagram(\"" << diagram.name() <<"\", diagram);" << endl;
	output << "\t\t};" << endl;
	output << "\t\t~_regClass()" << endl;

	output << "\t\t{" << endl;
	output <<"\t\t\tUdm::MetaDepository::RemoveDiagram(\"" << diagram.name() << "\");" << endl;
	output <<"\t\t};" <<endl;
	output <<"\t} __regUnUsed;" << endl << endl;

};
void GenerateCPPInitialize(const Uml::Diagram &diagram, const Uml::Diagram & cross_dgr, ostream & output, const string& macro, const string& hname, int ass_inits, int comp_inits)
{
	output << "\t" << macro << " void Initialize()" << endl << "\t{" << endl; //begin of Initialize()

	output << "\t\tstatic bool first = true;" << endl;
	output << "\t\tif(!first) return;" << endl;
	output << "\t\tfirst = false;" << endl;

	output << "\t\tUml::Initialize();" << endl << endl;
	if (cross_dgr && (diagram != cross_dgr))
		output << "\t\t" << cross_dgr.name() << "::Initialize();" << endl << endl;

	output << "\t\tASSERT( umldiagram == Udm::null );" << endl;

	output << "\t\tUdmStatic::StaticDataNetwork * meta_dn = new UdmStatic::StaticDataNetwork(Uml::diagram);" << endl;
	output << "\t\tmeta_dn->CreateNew(\"" << hname << ".mem\", \"\", Uml::Diagram::meta, Udm::CHANGES_LOST_DEFAULT);"<< endl;
	output << "\t\tumldiagram = Uml::Diagram::Cast(meta_dn->GetRootObject());" << endl;
	output << "\t\tumldiagram.name() =\"" << hname << "\";" << endl;
	output << "\t\tumldiagram.version() =\"" << diagram.version() << "\";" << endl;

	output << "\t\tCreates();" << endl;
	output << "\t\tInitClassesAttributes();" << endl;
		
	output << "\t\tUml::InitDiagram(umldiagram, \"" << hname << "\", \"" << diagram.version() << "\");" << endl << endl;
		
	
	//call the InitAssociation functions
		
	for (int ass_inits_i = 0; ass_inits_i < ass_inits; ass_inits_i++) output << "\t\tInitAssociation" << ass_inits_i << "();" << endl;
			
	//call the InitComposition calls

	for (int  comp_inits_i = 0; comp_inits_i < comp_inits; comp_inits_i++)	output << "\t\tInitComposition" << comp_inits_i << "();" << endl;

	//call the intializers for association roles created in the cross diagrams
	GenerateCPPCrossObjectInits(diagram, cross_dgr, output);

	//initialize inheritence
	GenerateCPPInitializeInheritence(diagram, output);		


	output << "\t\t" << endl << "\t}" << endl;	//end of Initialize()

};

/*
	Function building the alternate Initialize(const Uml::Diagram &dgr) function
*/

void GenerateCPPInitializeDiagram(const Uml::Diagram & dgr, ostream & output, const string & macro)
{
	set<Uml::Class> classes = dgr.classes();
	set<Uml::Class>::iterator c;
	
	output << "\t" << macro << " void Initialize(const Uml::Diagram & dgr)" << endl << "\t{" << endl;

	output << "\t\t" << "umldiagram = dgr;" << endl;
	for (c = classes.begin(); c != classes.end(); c++)
	{
		Uml::Class cl = *c;
		output << "\t\tUml::SetClass(" << cl.name() <<"::meta, umldiagram, \"" << cl.name() << "\");" <<endl;

		//attributes
		set<Uml::Attribute> cl_atts = cl.attributes();
		for (set<Uml::Attribute>::iterator cla_i = cl_atts.begin(); cla_i != cl_atts.end(); cla_i++)
		{
			Uml::Attribute a = *cla_i;
			output << "\t\tUml::SetAttribute(" << cl.name() <<"::meta_" << a.name() << "," << cl.name() << "::meta, \"" << a.name() << "\");" <<endl;

		};
	};

	output << "\t\t" << "//composition child roles " << endl;
	for (c = classes.begin(); c != classes.end(); c++)
	{
	
		Uml::Class cl = *c;

		//composition parent roles
		set<Uml::CompositionChildRole> ccrs = cl.childRoles();
		for (set<Uml::CompositionChildRole>::iterator ccrs_i = ccrs.begin(); ccrs_i != ccrs.end(); ccrs_i ++)
		{
			Uml::CompositionChildRole ccr = *ccrs_i;
			string rolename = Uml::MakeRoleName(Uml::theOther(ccr));
			string theot_name = ((Uml::Class)Uml::theOther(ccr).target()).name();
			output << "\t\tUml::SetParentRole(" << cl.name() <<"::meta_" << rolename << "," << cl.name() << "::meta," << theot_name << "::meta, \"" << ccr.name() << "\", \"" << Uml::theOther(ccr).name() << "\");" <<endl;

		};

	};
	
	output << "\t\t" << "// composition parentroles " << endl;
	for (c = classes.begin(); c != classes.end(); c++)
	{
		Uml::Class cl = *c;

		//composition child roles
		set<Uml::CompositionParentRole> cprs = cl.parentRoles();
		for (set<Uml::CompositionParentRole>::iterator cprs_i = cprs.begin(); cprs_i != cprs.end(); cprs_i ++)
		{
			Uml::CompositionParentRole cpr = *cprs_i;
			string rolename = Uml::MakeRoleName(Uml::theOther(cpr));
			string theot_name = ((Uml::Class)Uml::theOther(cpr).target()).name();
			output << "\t\tUml::SetChildRole(" << cl.name() <<"::meta_" << rolename << "," << cl.name() << "::meta," << theot_name << "::meta, \"" << cpr.name() << "\", \"" << Uml::theOther(cpr).name() << "\");" <<endl;

		};
	};
	
	output << "\t\t" << "// Association roles " << endl;
	for (c = classes.begin(); c != classes.end(); c++)
	{
		Uml::Class cl = *c;

		//association roles
		set<Uml::AssociationRole> ars = cl.associationRoles();
		for (set<Uml::AssociationRole>::iterator ars_i = ars.begin(); ars_i != ars.end(); ars_i ++)
		{
			Uml::AssociationRole ar = *ars_i;
			string rolename = Uml::MakeRoleName(Uml::theOther(ar));
			string theot_name = ((Uml::Class)Uml::theOther(ar).target()).name();

			output << "\t\tUml::SetAssocRole(" << cl.name() <<"::meta_" << rolename << "," << cl.name() << "::meta," << theot_name << "::meta, \"" << ar.name() << "\");" <<endl;

			Uml::Association as = ar.parent();
			Uml::Class aclass = as.assocClass();
			if (aclass)
			{
				string revrolename = Uml::MakeRoleName(ar) + "_rev";
				//set the _rev -s
				output << "\t\t" << ((Uml::Class)Uml::theOther(ar).target()).name() << "::meta_" << revrolename <<" = " << cl.name() << "::meta_" << rolename <<";" << endl;
			}
		};
	}
	for (c = classes.begin(); c != classes.end(); c++)
	{
		Uml::Class cl = *c;

		//_end_ association roles
		Uml::Association as = cl.association();
		if (as)
		{
			set<Uml::AssociationRole> ars = as.roles();
			for (set<Uml::AssociationRole>::iterator ars_i = ars.begin();ars_i != ars.end(); ars_i++)
			{
				Uml::AssociationRole ar = *ars_i;
				output << "\t\t" << cl.name() << "::meta_" << Uml::MakeRoleName(ar) << "_end_ = " << ((Uml::Class)Uml::theOther(ar).target()).name() << "::meta_" <<Uml::MakeRoleName(ar) << ";" << endl;

			};
		};
		output << endl;
	};
	
	output << "\t\t" << endl << "\t}" << endl; //end of Initialize(const Uml::Diagram & dgr);
};

void GenerateNewCPP(const Uml::Diagram &diagram,  ostream &output, string fname, const Uml::Diagram& cross_dgr = NULL, const string& macro = "") 
{
		string hname = diagram.name();
		string::iterator i;
		for(i = hname.begin(); i != hname.end(); i++) 
		{
			if(!isalnum(*i)) *i = '_';
		}

		output << "// cpp(meta datanetwork format) source file " << fname << ".cpp generated from diagram " << (string)diagram.name() << endl;
		output << "// generated on " << GetTime().c_str() << endl << endl;
		output << "#include \""<< fname << ".h\"" <<endl;
	

		output << "#include \"UmlExt.h\"" <<endl << endl; 
		output << "#include \"UdmStatic.h\"" << endl << endl; 

		if (cross_dgr && (diagram != cross_dgr))
		{
			output << "// cross-package metainformation header file" << endl;
			output << "#include \"" << cross_dgr.name() << ".h\"" << endl << endl; 
		}


		output << "namespace " << hname << " {" << endl << endl;
//	Uml::Diagram diagram;
		output << "\tUml::Diagram umldiagram;" << endl << endl;

		GenerateCPPDeclareMetaClasses(diagram, output);
		GenerateCPPDeclareAttributes(diagram, output);
		GenerateCPPDeclareAssociationRoles(diagram, cross_dgr, output);
		GenerateCPPDeclareCompositionRoles(diagram, output);
		GenerateCPPDeclareConstraints(diagram, output);
		GenerateCPPCreatesFunction(diagram, output);
		GenerateCPPInitClassesAttributes(diagram, cross_dgr, output);
	
		int ass_inits = GenerateCPPAssociationInitFunctions(diagram, output);		
		int comp_inits = GenerateCPPCompositionInitFunctions(diagram, output);
		
		//generate Initialize() function
		GenerateCPPInitialize(diagram, cross_dgr, output, macro, hname, ass_inits, comp_inits);
		
		//generate Initialize(const Uml::Diagram &dgr); function
		GenerateCPPInitializeDiagram(diagram, output,macro);

		
		//Udm::UdmDiagram diagram = { &umldiagram, Initialize };
		output << "\t" << macro << " Udm::UdmDiagram diagram = { &umldiagram, Initialize };" << endl;
		
		//generate the registering static class
		GenerateCPPStaticClass(diagram, output);

		//end of the namespace
		output << "}" << endl <<"// END " << hname << ".cpp" <<endl;
}



