/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

/*
CHANGELOG:

	02/14/05	-	endre
		
		fixed g++ 3.0 compiler error:
		UdmH.cpp: In constructor `InheritenceSolver::InheritenceSolver(const Uml::Diagram&)':
		UdmH.cpp:70: error: name lookup of `c' changed for new ISO `for' scoping
		UdmH.cpp:58: error:   using obsolete binding at `c'	

	12/20/04	-	endre

		took the functions from UdmCppH.cpp
		Also separated the huge generator function into a bunch of smaller functions to increase readability

*/



#include "Uml.h"
#include "UmlExt.h"
#include <time.h>
#include <map>
string GetTime();
using namespace Uml;
typedef vector<Uml::Class> UmlClasses;


class InheritenceSolver
{
	friend void GenerateH(const Uml::Diagram &diagram, ostream &output, string fname, bool visitor_sup, const Uml::Diagram& cross_dgr, const string& macro);
	typedef map< Uml::Class, set<Uml::Class> > cltoclsmap;
	typedef map< Uml::Class, bool > cltoboolmap;

	cltoclsmap virtualchildren;
	cltoclsmap explicitinits;
	cltoboolmap objectneeded_map;
	set<Uml::Class> virtualbaseclasses;
	vector<Uml::Class> good_inheritence_order;
		
public:

	InheritenceSolver(const Uml::Diagram& dgr)
	{
		set<Uml::Class> classes;
		classes = dgr.classes();
		while (!classes.empty())
		{

			Uml::Class cl;
			set<Uml::Class> bases;
			set<Uml::Class>::iterator c;
			for(c = classes.begin(); c != classes.end(); c++ )	
			{
				cl = *c;
				bases = cl.baseTypes();

				set<Uml::Class>::iterator i;
				for(i = bases.begin(); i != bases.end(); i++) 
				{
					if(classes.find(*i) != classes.end()) break;
				}
				if(i == bases.end()) break;  // this class had all its parents already printed
			}
			if(c == classes.end()) 
			{ // list is not empty, still we found no one to continue with
				throw udm_exception("inheritance circularity detected");
			}

			classes.erase(cl);
			good_inheritence_order.push_back(cl);

			virtualchildren.insert(cltoclsmap::value_type(cl, set<Uml::Class>()));
			
			//filling up virtual base classes
			if( bases.size() == 0) 
			{
// Object should be virtual, if any of the descendants has an immediate baseclass,
// that is not derived from the current class   (only descendants with >=2 baseclasses are to be checked)
				set<Uml::Class> descs = Uml::DescendantClasses(cl);
				for(set<Uml::Class>::iterator i = descs.begin(); i != descs.end(); i++) 
				{
					if(*i == cl) continue;
					set<Uml::Class> dirancs = i->baseTypes();
					if(dirancs.size() > 1) 
					{
						for(set<Uml::Class>::iterator j = dirancs.begin(); j != dirancs.end(); j++) 
						{
							if(!Uml::IsDerivedFrom(*j, cl)) 
							{
								virtualbaseclasses.insert(cl);
								break;
							}
						}
					}
				}
			}
			else 
			{
				set<Uml::Class>::iterator h;
				for(h = bases.begin(); h != bases.end(); h++) 
				{
// the current base should be virtual, if any of the descendants has an immediate baseclass,
// that is derived from the current base, but not from the current class 
// (only descendants with >=2 baseclasses are to be checked)
					cltoclsmap::iterator actmapi = virtualchildren.find(*h);
					if(actmapi == virtualchildren.end()) 
					{
						throw udm_exception("internal error in class ordering");
					}
					set<Uml::Class> descs = Uml::DescendantClasses(cl);
					set<Uml::Class>::iterator i;
					for( i = descs.begin(); i != descs.end(); i++) 
					{
						if(*i == cl) continue;
						set<Uml::Class> dirancs = i->baseTypes();
						if(dirancs.size() > 1) 
						{
							for(set<Uml::Class>::iterator j = dirancs.begin(); j != dirancs.end(); j++) 
							{
								if(!Uml::IsDerivedFrom(*j, cl) && Uml::IsDerivedFrom(*j, *h)) 
								{
									
										actmapi->second.insert(cl);
								}
							}
						}
					}

					descs = Uml::AncestorClasses(cl);
					for(i = descs.begin(); i != descs.end(); i++) 
					{
						if(*i == cl) continue;
						if (actmapi->second.find(*i) != actmapi->second.end())
						{
							actmapi->second.insert(cl);
							break;
						};
					}
				};
			};

			int object_needed = 0, object_notneeded = 0;
			set<Uml::Class> ancestors = Uml::AncestorClasses(cl); 
			set<Uml::Class> additionaexplicitinitializers;

			for(set<Uml::Class>::iterator i = ancestors.begin(); i != ancestors.end(); i++) 
			{
				if(*i == cl) continue;

				if(set<Uml::Class>(i->baseTypes()).size() == 0) 
				{
					if(virtualbaseclasses.find(*i) != virtualbaseclasses.end()) object_needed++;
					else object_notneeded++;
				}
				if(bases.find(*i) != bases.end()) continue;
			
				cltoclsmap::iterator actmapi = virtualchildren.find(*i);
				set<Uml::Class> subtypes = i->subTypes();
				int found_inset = 0, found_notinset = 0;
				for(set<Uml::Class>::iterator p = subtypes.begin(); p != subtypes.end(); p++) 
				{
				
					if(!Uml::IsDerivedFrom(cl, *p)) continue;
					if(actmapi->second.find(*p) != actmapi->second.end()) found_inset++;
					else found_notinset++;
				}
				if(found_notinset > 1 || (found_inset && found_notinset))
				{
					throw udm_exception(string("Internal error resolving inheritance from class ") + string(i->name()) + " to " + string(cl.name()));
				}
				else if(found_inset) 
				{
						additionaexplicitinitializers.insert(*i);
				}
			}
			if(object_notneeded > 1 || (object_needed && object_notneeded)) 
			{ 
					throw udm_exception("Internal error resolving inheritance from class Object to " + string(cl.name()));
			}
			
			cltoclsmap::value_type item1(cl, additionaexplicitinitializers);
			cltoboolmap::value_type item2(cl, (object_needed!=0));

			pair<cltoclsmap::iterator, bool> res1 = explicitinits.insert(item1);
			pair<cltoboolmap::iterator, bool> res2 = objectneeded_map.insert(item2);

			if (!res1.second || !res2.second)
				throw udm_exception(string("Error generating the explicit initializer lists for class:") + (string)cl.name());
		};


	};

	string getAncestorList(const Uml::Class &cl)
	{
		string ret;
		set<Uml::Class> bases = cl.baseTypes();
		if( bases.size() == 0) 
		{
			if (virtualbaseclasses.find(cl) != virtualbaseclasses.end()) ret+= " virtual";
			ret +=  " public Udm::Object";
		}
		else 
		{
			string sep("");
			set<Uml::Class>::iterator h;
			for(h = bases.begin(); h != bases.end(); h++) 
			{
				cltoclsmap::iterator actmapi = virtualchildren.find(*h);
				if(actmapi == virtualchildren.end()) 
				{
					throw udm_exception("internal error in class ordering");
				}
				if (actmapi->second.find(cl) != actmapi->second.end())
				{
					ret += sep;
					ret += " virtual";
					sep = "";
				};
				if(*h != cl) 
				{
					ret += sep;
					ret += " public ";
					ret += h->name();
					sep = ",";
				}
			}
		};
		return ret;
	};
	string getInitializers(const Uml::Class &cl, const string& argument)
	{
		set<Uml::Class> bases = cl.baseTypes();
		string ret;

		if( bases.size() == 0 ) 
		{
			ret +=  "UDM_OBJECT(";
			ret += argument;
			ret += ")";
		}
		else 
		{
			string sep("");
			set<Uml::Class>::iterator i;
			for(i = bases.begin(); i != bases.end(); i++) 
			{
				ret += sep;
				ret += i->name();
				ret += "(";
				ret += argument;
				ret += ")";

				sep = ",";
			}
		}

		cltoclsmap::iterator ei_i = explicitinits.find(cl);
		if (ei_i == explicitinits.end())
			throw udm_exception("InheritenceSolver error");

		for(set<Uml::Class>::iterator i = ei_i->second.begin(); i != ei_i->second.end(); i++) 
		{
			ret += ", ";
			ret += i->name();
			ret += "(";
			ret += argument;
			ret += ")";
		}

		cltoboolmap::iterator on_i = objectneeded_map.find(cl);
		if (on_i == objectneeded_map.end())
			throw udm_exception("InheritenceSolver error");

		if(on_i->second) 
		{
			ret += ", ";
			ret += "UDM_OBJECT(";
			ret += argument;
			ret += ")";
		}

		return ret;
	};

};

string CheckAttrType(const Udm::StringAttr &p) 
{

	string pv = p;
	if(pv.compare("String") && pv.compare("Boolean") && pv.compare("Integer") && pv.compare("Real")&& pv.compare("Text")) 
	{
		throw udm_exception("invalid attribute type '"+pv+"'");
	}

	//we use the same API for Text and String attributes
	if (!pv.compare("Text"))
	{
		return "String";
	};
	return pv;
};



void GenerateHClassConstraints(const Uml::Class &cl, ostream &output)
{
	//constraints
	set<Uml::Constraint> constraints = cl.constraints();
	set<Uml::Constraint>::iterator c_i = constraints.begin();
	while (c_i != constraints.end())
	{
		output << "\t\tstatic Uml::Constraint meta_" << c_i->name() <<";" << endl;
		c_i++;
	}

	//constraint definitions
	set<Uml::ConstraintDefinition> definitions= cl.definitions();
	set<Uml::ConstraintDefinition>::iterator d_i = definitions.begin();
	while (d_i != definitions.end())
	{
		output << "\t\tstatic Uml::ConstraintDefinition meta_" << d_i->name() <<";" << endl;
		d_i++;
	}
};
void GenerateHClassBasic(const Uml::Class & cl, InheritenceSolver& is, ostream & output, const string & hname)
{
	output << "\t public:" << endl;
	output << "\t\tstatic Uml::Class meta;" << endl << endl;

	output << "\t\t" << cl.name() << "() { }" << endl;
	output << "\t\t" << cl.name() << "(Udm::ObjectImpl *impl) : " << is.getInitializers(cl, "impl");
	output << " { }" << endl;

	output << "\t\t" << cl.name() << "(const " << cl.name() << " &master) : " << is.getInitializers(cl, "master");
	output << " { }" << endl;

			
	output << "\t\tstatic " << cl.name() << " Cast(const Udm::Object &a) { return __Cast(a, meta); }" << endl << endl;
	output << "\t\tstatic " << cl.name() << " Create(const Udm::Object &parent, const Uml::CompositionChildRole &role = Udm::NULLCHILDROLE)"
			" { return __Create(meta, parent, role); }" << endl << endl;

	output << "\t\t" << cl.name() << " CreateInstance(const Udm::Object &parent, const Uml::CompositionChildRole &role = Udm::NULLCHILDROLE)"
			" { return __Create(meta, parent, role, impl); }" << endl << endl;

	output << "\t\t" << cl.name() << " CreateDerived(const Udm::Object &parent, const Uml::CompositionChildRole &role = Udm::NULLCHILDROLE)"
			" { return __Create(meta, parent, role, impl, true); }" << endl << endl;

	output << "\t\t" << "Udm::InstantiatedAttr<" << (string)cl.name() << "> Instances()"
			" { return Udm::InstantiatedAttr<" <<hname << "::" << (string)cl.name() << ">(impl);}" <<endl;

	output << "\t\t" << "template <class Pred> Udm::InstantiatedAttr<" << (string)cl.name() << ", Pred> Instances_sorted(const Pred &)"
			" { return Udm::InstantiatedAttr<" <<hname << "::" << (string)cl.name() << ", Pred>(impl);}" <<endl <<endl;


	output << "\t\t" << "Udm::DerivedAttr<" << hname << "::" << (string)cl.name() << "> Derived()"
			" { return Udm::DerivedAttr<" <<hname << "::"<< (string)cl.name() << ">(impl);}" <<endl ;

	output << "\t\t" << "template <class Pred> Udm::DerivedAttr<" << hname << "::" << (string)cl.name() << ", Pred> Derived_sorted(const Pred &)"
			" { return Udm::DerivedAttr<" <<hname << "::"<< (string)cl.name() << ", Pred>(impl);}" <<endl <<endl;


	output << "\t\t" << "Udm::ArchetypeAttr<" <<hname << "::" << (string)cl.name() << "> Archetype()"
			" { return Udm::ArchetypeAttr<" <<hname << "::" << (string)cl.name() << ">(impl);}" <<endl <<endl;

};

void GenerateHClassAssocEnds(const Uml::Class &cl, const Uml::Class& cross_cl, ostream & output, const string &hname, bool isCrossDgr)
{
	Uml::Association assoc = cl.association();
	string s = assoc.meta.name();
	if(assoc) 
	{
		set<Uml::AssociationRole> assocs = assoc.roles();
		for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
		{
			string aname(MakeRoleName(*i));
			string oname(((Uml::Class)(*i).target()).name());
			if(aname.size()) 
			{
			      output << "\t\tstatic Uml::AssociationRole meta_" << aname << "_end_;" << endl;
			      output << "\t\tUdm::AssocEndAttr<" << hname << "::" << oname << "> " << aname << 
					  "_end() const { return Udm::AssocEndAttr<"<< hname << "::" << oname << ">(impl, meta_" << aname << "_end_); }\n\n";
			}
		}
	}
	if (cross_cl && !isCrossDgr)
	{
		Uml::Association cross_cl_assoc = cross_cl.association();
		string s = cross_cl_assoc.meta.name();
		if (cross_cl_assoc)
		{
			set<Uml::AssociationRole> assocs = cross_cl_assoc.roles();
			for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
			{
				string aname(MakeRoleName(*i));
				string oname(((Uml::Class)(*i).target()).name());
				//Attention !
				oname = oname.substr(0, oname.find(Udm::cross_delimiter));
				//end of dangerous zone 
				string from(((Uml::Class)(*i).target()).from());
				if(aname.size()) 
				{
				      output << "\t\tstatic Uml::AssociationRole meta_" << aname << "_end_;" << endl;
				      output << "\t\tUdm::CrossAssocEndAttr<" << from << "::" << oname << "> " << aname << 
					  "_end() const { return Udm::CrossAssocEndAttr<"<< from<< "::" << oname << ">(impl, meta_" << aname << "_end_); }\n\n";
				}
			}
		}
	}
};
void GenerateHClassParents(const Uml::Class &cl, ostream& output, const string &hname)
{
	set<Uml::CompositionChildRole> children = cl.childRoles();
	for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) 
	{
		string aname(MakeRoleName(Uml::theOther(*i)));
		string oname(((Uml::Class)Uml::theOther(*i).target()).name());
		output << "\t\tstatic Uml::CompositionParentRole meta_" << aname << ";" << endl;
		if(Uml::theOther(*i).isNavigable()) 
		{
			output << "\t\tUdm::" << "Parent" << "Attr<" << hname << "::" << oname << "> " << aname << 
			"() const { return Udm::" << "Parent" << "Attr<" << hname << "::" << oname << 
			">(impl, meta_" << aname << "); }\n\n";
		}
		else 
		{
		  output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
		}
	}
	bool parent_defined = false;
	set<Uml::CompositionParentRole> parroles = Uml::AncestorCompositionPeerParentRoles(cl);
	for( set<Uml::CompositionParentRole>::iterator ri = parroles.begin(); ri != parroles.end(); ri++) 
	{
		if(string(ri->name()) == "parent") parent_defined = true;
	}
	if(!parent_defined) 
	{
		set<Uml::Class> ans = Uml::CommonAncestorClasses(Uml::AncestorContainerClasses(cl));
		if(ans.empty()) 
		{
			output << "\t\tUdm::ParentAttr<Udm::Object> parent() const " << "{ return Udm::ParentAttr<Udm::Object>(impl, Udm::NULLPARENTROLE); }" << endl;
		}
		else 
		{
			if (ans.size() == 1)
			{
				output << "\t\tUdm::ParentAttr<" << hname << "::" << (string)ans.begin()->name() << "> parent() const " << 
				"{ return Udm::ParentAttr<" << hname << "::" << (string)ans.begin()->name() << ">(impl, Udm::NULLPARENTROLE); }" << endl;
			}
			else
			{
				for(set<Uml::Class>::iterator ccc = ans.begin(); ccc != ans.end(); ccc++) 
				{
					output << "\t\tUdm::ParentAttr<" << hname << "::" << (string)ccc->name() << "> " << (string)ccc->name() << "_parent() const " << 
					"{ return Udm::ParentAttr<" << hname << "::" << (string)ccc->name() << ">(impl, Udm::NULLPARENTROLE); }" << endl;
				}
			}
		}
	}
};
void GenerateHClassChildren(const Uml::Class &cl, const Uml::Diagram & dgr, ostream & output, const string & hname)
{

	set<Uml::Class> childrenkinds; 
	set<Uml::CompositionParentRole> children = cl.parentRoles();
	for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) 
	{
		Uml::CompositionParentRole k;
		string aname(MakeRoleName(Uml::theOther(*i)));
		Uml::Class thischildkind = (Uml::Class)Uml::theOther(*i).target();
		childrenkinds.insert(thischildkind);
		string oname(thischildkind.name());
		output << "\t\tstatic Uml::CompositionChildRole meta_" << aname << ";" << endl;
		if(Uml::theOther(*i).isNavigable()) 
		{
			if ((int)Uml::theOther(*i).max() == 1 )
			{
				output << "\t\tUdm::ChildAttr<" << hname << "::" << oname << "> " << aname << 
				"() const { return Udm::ChildAttr<" << hname << "::" << oname << 
				">(impl, meta_" << aname << "); }\n\n";
			}
			else
			{
				output << "\t\tUdm::ChildrenAttr<" << hname << "::" << oname << "> " << aname << 
				"() const { return Udm::ChildrenAttr<" << hname << "::" << oname << 
				">(impl, meta_" << aname << "); }\n";

				output << "\t\t template <class Pred> Udm::ChildrenAttr<" << hname << "::" << oname << ", Pred> " << aname << 
				"_sorted(const Pred &) const { return Udm::ChildrenAttr<" << hname << "::" << oname << 
				", Pred>(impl, meta_" << aname << "); }\n\n";
			}
		}
		else 
		{
		     output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
		}
	}


	set<Uml::Class> allclasses = dgr.classes();
	for(set<Uml::Class>::iterator j = allclasses.begin(); j != allclasses.end(); j++) 
	{
		for(set<Uml::Class>::iterator k = childrenkinds.begin(); k != childrenkinds.end(); k++) 
		{
			if(Uml::IsDerivedFrom(*j, *k) || Uml::IsDerivedFrom(*k, *j)) 
			{
				output << "\t\tUdm::ChildrenAttr<" << hname << "::" << (*j).name() << "> " << (*j).name() << 
				"_kind_children() const { return Udm::ChildrenAttr<" << hname << "::" << (*j).name() <<
				">(impl, Udm::NULLCHILDROLE); }\n";

				output << "\t\ttemplate<class Pred> Udm::ChildrenAttr<" << hname << "::" << (*j).name() << ", Pred> " << (*j).name() << 
				"_kind_children_sorted(const Pred &) const { return Udm::ChildrenAttr<" << hname << "::" << (*j).name() <<
				", Pred>(impl, Udm::NULLCHILDROLE); }\n\n";

				break;
			}
		}
	}
};

void GenerateHClassAssociations(const Uml::Class& cl, const Uml::Class& cross_cl, ostream & output, const string& hname, bool isCrossDgr)
{
	set<Uml::AssociationRole> assocs = cl.associationRoles();
	for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
	{
		Uml::Association pp = (*i).parent();
		Uml::Class aclass = pp.assocClass();

		string aname(MakeRoleName(Uml::theOther(*i)));
		string oname(((Uml::Class)Uml::theOther(*i).target()).name());
		if(!aclass) 
		{
			output << "\t\tstatic Uml::AssociationRole meta_" << aname << ";" << endl;
			if(Uml::theOther(*i).isNavigable()) 
			{
				if (Uml::theOther(*i).max() == 1)
				{
					output << "\t\tUdm::PointerAttr<" << hname << "::" << oname << "> " << aname << 
					"() const { return Udm::PointerAttr<"<< hname << "::" << oname << 
					">(impl, meta_" << aname << "); }\n\n";
					
				}
				else
				{
					output << "\t\tUdm::AssocAttr<" << hname << "::" << oname << "> " << aname << 
					"() const { return Udm::AssocAttr<"<< hname << "::" << oname << 
					">(impl, meta_" << aname << "); }\n";

					output << "\t\ttemplate <class Pred> Udm::AssocAttr<" << hname << "::" << oname << ", Pred > " << aname << 
					"_sorted(const Pred &) const { return Udm::AssocAttr<"<< hname << "::" << oname << 
					", Pred>(impl, meta_" << aname << "); }\n\n";					
				}
			}
			else 
			{
				output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
			}
		}
		else 
		{
			string clname(aclass.name());
			output << "\t\tstatic Uml::AssociationRole meta_" << aname << ";" << endl;
			output << "\t\tstatic Uml::AssociationRole meta_" << aname << "_rev;" << endl;
			if(Uml::theOther(*i).isNavigable()) 
			{
				if (Uml::theOther(*i).max() == 1)
				{
					output << "\t\tUdm::AClassPointerAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << "> " << aname << 
					"() const { return Udm::AClassPointerAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << 
					">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
				}
				else
				{	
					output << "\t\tUdm::AClassAssocAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << "> " << aname << 
					"() const { return Udm::AClassAssocAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << 
					">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n";

					output << "\t\ttemplate<class Pred> Udm::AClassAssocAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << ", Pred> " << aname << 
					"_sorted(const Pred &) const { return Udm::AClassAssocAttr<"<< hname << "::" << clname << ", " << hname << "::" << oname << 
					", Pred>(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
				}
			}
			else 
			{
					output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
			}
		}
	}
	if (cross_cl && !isCrossDgr)
	{
		set<Uml::AssociationRole> assocs = cross_cl.associationRoles();
		for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) 
		{
			Uml::Association pp = (*i).parent();
			Uml::Class aclass = pp.assocClass();

			string aname(MakeRoleName(Uml::theOther(*i)));
			string oname(((Uml::Class)Uml::theOther(*i).target()).name());
			//Attention !!!
			oname = oname.substr(0,oname.find(Udm::cross_delimiter));
			//
			string cname(((Uml::Class)Uml::theOther(*i).target()).from());

			if(!aclass) 
			{
				output << "\t\tstatic Uml::AssociationRole meta_" << aname << ";" << endl;
				if(Uml::theOther(*i).isNavigable()) 
				{
                    if (Uml::theOther(*i).max() == 1 )
					{
						output << "\t\tUdm::CrossPointerAttr<" << cname << "::" << oname << "> " << aname << 
						"() const { return Udm::CrossPointerAttr<"<< cname << "::" << oname << 
						">(impl, meta_" << aname << "); }\n\n";
					}
					else
					{
						output << "\t\tUdm::CrossAssocAttr<" << cname << "::" << oname << "> " << aname << 
						"() const { return Udm::CrossAssocAttr<"<< cname << "::" << oname << 
						">(impl, meta_" << aname << "); }\n";

						output << "\t\ttemplate<class Pred> Udm::CrossAssocAttr<" << cname << "::" << oname << ", Pred> " << aname << 
						"_sorted(const Pred &) const { return Udm::CrossAssocAttr<"<< cname << "::" << oname << 
						", Pred>(impl, meta_" << aname << "); }\n\n";
					};
				}
				else 
				{
					output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
				};
			}
			else 
			{
				string clname(aclass.name());
				//Attention !!!
				clname = clname.substr(0,clname.find(Udm::cross_delimiter));
				//end of dangerous zone
	
				string cl_dgr(aclass.from());
				output << "\t\tstatic Uml::AssociationRole meta_" << aname << ", meta_" << aname << "_rev;" << endl;
				if(Uml::theOther(*i).isNavigable()) 
				{
					if (Uml::theOther(*i).max() == 1)
					{
						output << "\t\tUdm::AClassCrossPointerAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << "> " << aname << 
						"() const { return Udm::AClassCrossPointerAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << 
						">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
					}
					else
					{
						output << "\t\tUdm::AClassCrossAssocAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << "> " << aname << 
						"() const { return Udm::AClassCrossAssocAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << 
						">(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n";
						output << "\t\ttemplate<class Pred> Udm::AClassCrossAssocAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << ", Pred> " << aname << 
						"_sorted(const Pred &) const { return Udm::AClassCrossAssocAttr<"<< cl_dgr << "::" << clname << ", " << cname << "::" << oname << 
						", Pred>(impl, meta_" << aname << ", meta_" << aname << "_rev); }\n\n";
					};
				}
				else 
				{
					  output << "\t\t// Access method for non-navigable association " << aname << " omitted \n\n";
				};
			};
		};
	};
};
void GenerateHClassAttributes(const Uml::Class& cl, ostream & output)
{
	
//		static Uml::Attribute meta_d;
//		Udm::RealAttr d() const { return Udm::RealAttr(impl, meta_d); }
	set<Uml::Attribute> attributes = cl.attributes();
	for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
	{
		output << "\t\tstatic Uml::Attribute meta_" << (*i).name() << ";" << endl;
		Uml::Attribute k;
		string np;
		if(i->nonpersistent()) np = "Temp";
		/*
			Array support
			endre, 06/17/2002
		*/
		if ((i->max() ==1) || (i->max() == 0))
			output << "\t\tUdm::" << np << CheckAttrType(i->type()) << "Attr " << (*i).name() << 
			"() const { return Udm::" << np <<CheckAttrType(i->type()) << "Attr(impl, meta_" << (*i).name() << "); }\n\n";
		else 
			output << "\t\tUdm::" << np << CheckAttrType(i->type()) << "AttrArr " << (*i).name() << 
			"() const { return Udm::" << np << CheckAttrType(i->type()) << "AttrArr(impl, meta_" << (*i).name() << "); }\n\n";
	}
};
void GenerateHVisitorClass(const Uml::Diagram& dgr, ostream & output, const string& macro)
{
	set<Uml::Class> classes = dgr.classes();
	set<Uml::Class>::iterator c;

	output << endl << "\tclass " << macro << " Visitor : public Udm::BaseVisitor " << endl << "\t{" << endl << "\tpublic:" <<endl;
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		if (!cl.isAbstract())
			output << "\t\tvirtual void Visit_" << (string)cl.name() << "(const " << (string)cl.name() << "&){};" << endl;	
	}

	output << "\t\tvirtual void Visit_Object(const Udm::Object&){};" << endl;	
	output << "\t\tvirtual ~Visitor(){};" << endl;	
	output << "\t};" << endl;
};

void GenerateHCrossForwardDeclarations(const Uml::Diagram &diagram, ostream &output,  const Uml::Diagram& cross_dgr, const string& dgr_name, const string & macro)
{
	if (cross_dgr)
	{
		set<Uml::Class> cross_classes = cross_dgr.classes();
		map<string, UmlClasses> cr_cls_nsp;
		for(set<Uml::Class>::iterator i = cross_classes.begin(); i != cross_classes.end(); i++)
		{
			string from = i->from();
			if ((from == dgr_name)|| (from.size()==0)) continue;
			vector<Uml::Class> & clss = cr_cls_nsp[from];
			clss.push_back(*i);
		};
		for(map<string, UmlClasses>::iterator j = cr_cls_nsp.begin(); j != cr_cls_nsp.end(); j++)
		{
			output << "namespace " << j->first << endl << "{ " << endl;
			vector<Uml::Class>& clss = j->second;
			for(vector<Uml::Class>::iterator jj = clss.begin(); jj != clss.end(); jj++)
			{
				string class_name = jj->name();
				//Attention - dangerous zone
				class_name = class_name.substr(0, class_name.find(Udm::cross_delimiter));
				//end of dangerous zone
				output << "\t class " << macro << " " << class_name << ";" << endl;
			};
			output << "};" << endl;
		};
	};
};

void GenerateHExport(const Uml::Diagram &diagram, ostream &output, string fname,  const string& macro)
{
	string hname = diagram.name();
	string::iterator i;
	for(i = hname.begin(); i != hname.end(); i++) 
	{
		if(!isalnum(*i)) *i = '_';
	}
	
	string uhname = "MOBIES_" + fname + "_H";
	string hasdll_string = hname + "_HAS_DLL";
	string export_string = hname + "_EXPORT";
	
	output << "#ifndef " << uhname <<endl;
	output << "#	define " << uhname <<endl;
	output << "#	ifndef " << macro << endl;
	output << "#		if defined (" << hasdll_string << ") && (" << hasdll_string << " == 1)" <<endl;
	output << "#			if defined (" << export_string << " )" << endl;
	output << "#				define " << macro << " __declspec(dllexport)" << endl;
	output << "#			else" << endl;
	output << "#				define " << macro << " __declspec(dllimport)" << endl;
	output << "#			endif /* " << export_string << " */" << endl;
	output << "#		else" <<endl;
	output << "#			define " << macro << endl;
	output << "#		endif /*" << hasdll_string << "*/" << endl;
	output << "#	endif /* ifndef " << macro << " */ " << endl;
	output << "#endif /* " << uhname << " */ " << endl;



};

void GenerateH(const Uml::Diagram &diagram, ostream &output, string fname, bool visitor_sup, const Uml::Diagram& cross_dgr, const string& macro)	
{
		string hname = diagram.name();
		
		string::iterator i;
		for(i = hname.begin(); i != hname.end(); i++) {
			if(!isalnum(*i)) *i = '_';
		}
		string uhname = "MOBIES_" + fname + "_H";

		for(i = uhname.begin(); i != uhname.end(); i++) {
			*i = toupper(*i);
		}

		output << "#ifndef "<< uhname << endl <<"#define " << uhname << endl;
		output << "// header file " << fname << ".h generated from diagram " << (string)diagram.name() << endl;
		output << "// generated on " << GetTime().c_str() << endl << endl;


		output << "#ifndef MOBIES_UDMBASE_H" << endl;
		output << "#include \"UdmBase.h\""	<< endl;
		output << "#endif"	<< endl << endl;

		if (macro.size())
			output << "#include \""<< fname << "_export.h\"" <<endl;
		

		InheritenceSolver is(diagram);
		GenerateHCrossForwardDeclarations(diagram, output, cross_dgr, hname, macro);

		output << endl << endl;
		output << "namespace " << hname << " {" << endl << endl;
		
		set<Uml::Class> classes = diagram.classes();
		set<Uml::Class>::iterator c;

		for(c = classes.begin(); c != classes.end(); c++ )
		{
			Uml::Class cl = *c;
			output << "\tclass " << macro <<" "<< (string)cl.name() << ";" << endl;	
		}

		if (visitor_sup) GenerateHVisitorClass(diagram, output, macro);

		output << endl << "\t " << macro << " void Initialize();" << endl;
		output << endl << "\t " << macro << " void Initialize(const Uml::Diagram &dgr);" << endl;
		output <<"\textern " << macro << " Udm::UdmDiagram diagram;" << endl << endl;

		for(vector<Uml::Class>::iterator gio_i = is.good_inheritence_order.begin();gio_i != is.good_inheritence_order.end(); gio_i++)
		{
			Uml::Class cl, cross_cl;
		
			cl = *gio_i;
			if (cross_dgr)
			{
				//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
				cross_cl = Uml::classByName(cross_dgr, ((string)(cl.name()) + string(Udm::cross_delimiter) + (string)diagram.name()) );
			}

			output << "\tclass " << macro << " " << (string)cl.name() << " : " << is.getAncestorList(cl) << endl;

			output << "\t{" << endl;
			GenerateHClassBasic(cl, is, output, hname);
			GenerateHClassAttributes(cl, output);
			GenerateHClassAssociations(cl, cross_cl, output, hname, (diagram == cross_dgr));
			GenerateHClassChildren(cl, diagram, output, hname);
			GenerateHClassParents(cl, output, hname);
			GenerateHClassAssocEnds(cl, cross_cl, output, hname, (diagram == cross_dgr));
			GenerateHClassConstraints(cl, output);
			
			//visitor
			if (visitor_sup && !cl.isAbstract())
				output << "\t\tvirtual void Accept(Visitor &v){v.Visit_" << (string)cl.name() <<"(*this);};" << endl;
			output << "\t};" << endl << endl;
		}
		
		//visitor class
		
		
		output << "}\n\n#endif //" << uhname << endl;
}

