/*
Copyright (c) Vanderbilt University, 2000-2001
ALL RIGHTS RESERVED
Vanderbilt University disclaims all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall Vanderbilt University be liable for
any special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
*/

/*
CHANGELOG:

	12/20/04	-	endre

		took the functions from UdmCppH.cpp
		Also separated the huge generator function into a bunch of smaller functions to increase readability

*/




#include "Uml.h"
#include "UmlExt.h"
#include <time.h>
#include <map>
string GetTime();
using namespace Uml;
typedef vector<Uml::Class> UmlClasses;



/*
	Declarations in the namespace
	Defined in UdmCpp.cpp
*/
void GenerateCPPDeclareMetaClasses(const Uml::Diagram & dgr, ostream & output);
void GenerateCPPDeclareAttributes(const Uml::Diagram & dgr, ostream & output);
void GenerateCPPDeclareAssociationRoles(const Uml::Diagram &dgr, const Uml::Diagram &cross_dgr, ostream & output);
void GenerateCPPDeclareCompositionRoles(const Uml::Diagram &dgr, ostream& output);
void GenerateCPPDeclareConstraints(const Uml::Diagram &dgr, ostream & output);
void GenerateCPPStaticClass(const Uml::Diagram &diagram, ostream & output);
/*
	Functions used to generate Initialize()
	They are defined here.
*/
void GenerateMetaCPPCreates(const Uml::Diagram &diagram, ostream &output)
{
	set<Uml::Class> classes = diagram.classes();
	set<Uml::Class>::iterator c;
	//	A::meta = Uml::CreateClass();
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		output << "\t\t" << cl.name() << "::meta = Uml::CreateClass();" << endl;
	}

//	A::meta_a = Uml::CreateAttribute();
	for(c = classes.begin(); c != classes.end(); c++ )	
	{
		Uml::Class cl = *c;
//		Uml::Class tempcl(Uml::classByName(tempdiagram, cl.name()));
		set<Uml::Attribute> attributes = cl.attributes();
/*		if(tempcl) {
			set<Uml::Attribute> tatts = tempcl.attributes();
			attributes.insert(tatts.begin(), tatts.end());
		}*/
		for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) {
			output << "\t\t" << cl.name() << "::meta_" << (*i).name() << " = Uml::CreateAttribute();" << endl;
		}
	}

//	A::meta_dst = Uml::CreateAssociationRole();
	for(c = classes.begin(); c != classes.end(); c++ ) {
			Uml::Class cl = *c;
			set<Uml::AssociationRole> assocs = cl.associationRoles();
			for( set<Uml::AssociationRole>::iterator i = assocs.begin(); i != assocs.end(); i++) {
				string aname(MakeRoleName(Uml::theOther(*i)));
				output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::CreateAssociationRole();" << endl;
		}
	}

//	A::meta_parent = Uml::CreateCompositionParentRole();
	for(c = classes.begin(); c != classes.end(); c++ ) {
		Uml::Class cl = *c;
		set<Uml::CompositionChildRole> children = cl.childRoles();
		for( set<Uml::CompositionChildRole>::iterator i = children.begin(); i != children.end(); i++) {
			string aname(MakeRoleName(Uml::theOther(*i)));
			output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::CreateCompositionParentRole();" << endl;
		}
	}

//	A::meta_as = Uml::CreateCompositionChildRole();

	for(c = classes.begin(); c != classes.end(); c++ ) {
		Uml::Class cl = *c;
		set<Uml::CompositionParentRole> children = cl.parentRoles();
		for( set<Uml::CompositionParentRole>::iterator i = children.begin(); i != children.end(); i++) {
			string aname(MakeRoleName(Uml::theOther(*i)));
			output << "\t\t" << cl.name() << "::meta_" << aname << " = Uml::CreateCompositionChildRole();" << endl;
		}
	}
// A::meta_constraintname = Uml::CreateConstraint();
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::Constraint> constraints = cl.constraints();
		for( set<Uml::Constraint>::iterator i = constraints.begin(); i != constraints.end(); i++) 
			output << "\t\t" << cl.name() << "::meta_" << i->name() << " = Uml::CreateConstraint();" << endl;
	}

// A::meta_definitionname = Uml::CreateConstraintDefinition();
		
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::ConstraintDefinition> definitions = cl.definitions();
		for( set<Uml::ConstraintDefinition>::iterator i = definitions.begin(); i != definitions.end(); i++) 
			output << "\t\t" << cl.name() << "::meta_" << i->name() << " = Uml::CreateConstraintDefinition();" << endl;
	}
};



void GenerateMetaCPPInitMetaObjects(const Uml::Diagram &diagram, const Uml::Diagram& cross_dgr, ostream & output)
{
	set<Uml::Class> classes = diagram.classes();
	set<Uml::Class>::iterator c;

	//Initialization of classes
//	Uml::InitClass(A::meta, diagram, "A", false);
	for(c = classes.begin(); c != classes.end(); c++ )
	{
		Uml::Class cl = *c;
		string ster;
		if(((string)cl.stereotype()).length()) ster = string(", \"")+(string)cl.stereotype()+"\"";

		string from;
		if(((string)cl.from()).length()) from = string(", \"")+(string)cl.from()+"\"";

		output << "\t\tUml::InitClass(" << cl.name() << "::meta, umldiagram, \"" <<
			cl.name() << "\", " << (cl.isAbstract() ? "true" : "false")<< ster << from  <<");" << endl;

	}
	output << "\t\t" << endl;

	//intialization of attributes, constraints, constraindefinitions, cross association class's association ends.
//	Uml::InitAttribute(A::meta_a, A::meta, "a", "String");
	for(c = classes.begin(); c != classes.end(); c++ )	
	{
		Uml::Class cl = *c;
		set<Uml::Attribute> attributes = cl.attributes();
		for( set<Uml::Attribute>::iterator i = attributes.begin(); i != attributes.end(); i++) 
		{
			//reproduce the defval string array
			if (i->defvalue())
			{
				vector<string> dva = i->defvalue();
				output << "\t\tvector<string> " << cl.name() << "_" << (*i).name() << "_dva;" <<endl;;

				vector<string>::const_iterator dva_ci = dva.begin();
	
				while (dva_ci != dva.end())
				{
					output << "\t\t" << cl.name() << "_" << (*i).name() << "_dva.push_back(\"" << *dva_ci++ << "\");" << endl; 
				};
				output << "\t\tUml::InitAttribute(" << cl.name() << "::meta_" << (*i).name() << ", " <<
					cl.name() << "::meta, \"" << (*i).name() << "\", \"" << (*i).type() << "\", " <<
					((*i).nonpersistent() ? "true" : "false") << ", " << ((*i).registry() ? "true" : "false") << ", " << (*i).min() << ", " << (*i).max() << ", " <<  
					((*i).ordered() ? "true" : "false") << ", \"" << (*i).visibility() <<
					"\", " << cl.name() << "_" << (*i).name() << "_dva" << ");" << endl;
			}
			else
			{
				output << "\t\tUml::InitAttribute(" << cl.name() << "::meta_" << (*i).name() << ", " <<
					cl.name() << "::meta, \"" << (*i).name() << "\", \"" << (*i).type() << "\", " <<
					((*i).nonpersistent() ? "true" : "false") << ", " << ((*i).registry() ? "true" : "false") << ", " << (*i).min() << ", " << (*i).max() << ", " <<  
					((*i).ordered() ? "true" : "false") << ", \"" << (*i).visibility() <<
					"\");" << endl;
			}
		}

		//constraints
		//(const Constraint &obj, const Class &parent, const char * name, const char * description, const char * expression);
		set<Uml::Constraint> constraints = cl.constraints();
		for( set<Uml::Constraint>::iterator j = constraints.begin(); j != constraints.end(); j++) 
		{
			output << "\t\tUml::InitConstraint(" << cl.name() << "::meta_" << j->name() << ", " <<
				cl.name() << "::meta, \"" << j->name() <<"\", \"" << j->description() << "\", \"" <<
				j->expression() << "\");" << endl;;
		}


		//constraint definitions
		//(const ConstraintDefinition &obj, const Class &parent, const char * name, const char * stereotype, 
		//const char * parameterList, const char * expression, const char * returnType);
		set<Uml::ConstraintDefinition> defs = cl.definitions();
		for( set<Uml::ConstraintDefinition>::iterator k = defs.begin(); k != defs.end(); k++) 
		{
			output << "\t\tUml::InitConstraintDefinition(" << cl.name() << "::meta_" << k->name() << ", " <<
				cl.name() << "::meta, \"" << k->name() <<"\", \"" << k->stereotype() << "\", \"" <<
				k->parameterList() << "\", \"" << k->expression() << "\", \"" << k->returnType() << "\");" << endl ;
		}

		//cross diagram - setting up cross-associationroles
		if (cross_dgr &&(cross_dgr != diagram))
		{
			Uml::Class cross_cl;
			Uml::Association ass;
			if (cross_dgr)
			{
				//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
				cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name())) );
			}
			if (cross_cl)
				ass = cross_cl.association();
			if (ass)
			{
				set<Uml::AssociationRole> asrs = ass.roles();
				set<Uml::AssociationRole>::iterator asrs_i = asrs.begin();
				
				while (asrs_i != asrs.end())
				{
					output << " \t\t" << cl.name() << "::meta_" << asrs_i->name() << "_end_ = " 
						<< cross_dgr.name() << "::" << ((Uml::Class)((Uml::theOther(*asrs_i)).target())).name()<<"::meta_" << asrs_i->name() << ";" <<endl;
					asrs_i++;
				}
			};
		}
	}
	output << "\t\t" << endl;
};
void GenerateMetaCPPInitAssociations(const Uml::Diagram & diagram, ostream & output)
{
	//initialization of associations
	{
		const set<Uml::Association> &asss = diagram.associations();
		set<Uml::Association>::const_iterator a;
		for(a = asss.begin(); a != asss.end(); a++ )	
		{
			set<Uml::AssociationRole> ar = a->roles();
			ASSERT(ar.size() == 2);
			output << "\t\t{" << endl;
			output << "\t\t\tUml::Association ass = Uml::CreateAssociation();" << endl;

			Uml::Class aclass = a->assocClass();
			output << "\t\t\tUml::InitAssociation(ass , umldiagram, \"" << a->name() << "\");" << endl;
			if(aclass) 
			{
				output << "\t\t\tUml::InitAssociationClass(ass, " << aclass.name() << "::meta);" << endl;
			}

			set<Uml::AssociationRole> roles = a->roles();
			for( set<Uml::AssociationRole>::iterator i = roles.begin(); i != roles.end(); i++) 
			{
					Uml::AssociationRole zz	= Uml::theOther(*i);
					string aname(MakeRoleName(zz));
					Uml::Class cl = i->target();
					output << "\t\t\tUml::InitAssociationRole(" << cl.name() << "::meta_" << aname 
						<< ", ass, \"" << aname << "\", " <<
						(zz.isNavigable() ? "true, " : "false, ") <<
						(zz.isPrimary() ? "true, " : "false, ") <<
						zz.min() << ", " << zz.max() << ", " << ((Uml::Class)zz.target()).name() << "::meta);" << endl;
					Uml::Class aclass = a->assocClass();
					if(aclass) 
					{
						Uml::Class cl2 = zz.target();
						output << "\t\t\t" << aclass.name() << "::meta_" << aname << "_end_ = " <<
											cl2.name() << "::meta_" << MakeRoleName(*i) << "_rev = " << 
											cl.name() << "::meta_" << aname << ";" << endl;
					}
			}		
			output << "\t\t}" << endl;
		}
	}
};
void GenerateMetaCPPInitCompositions(const Uml::Diagram & diagram, ostream & output)
{
	//initialization of compositions
	const set<Uml::Composition> &comps = diagram.compositions();
	set<Uml::Composition>::const_iterator c;
	for(c = comps.begin(); c != comps.end(); c++ )	
	{
		output << "\t\t{" << endl;
		Uml::Composition comp = Uml::CreateComposition();
		output << "\t\t\tUml::Composition comp = Uml::CreateComposition();" << endl;

//		Uml::InitComposition(comp, diagram, "DtoA");
		output << "\t\t\tUml::InitComposition(comp , umldiagram, \"" <<  c->name() << "\");" <<  endl;
		output << "\t\t" << endl;

		Uml::CompositionParentRole x1 = (*c).parentRole();
		Uml::CompositionChildRole x2 = (*c).childRole();

		Uml::CompositionParentRole zzp = c->parentRole();
		Uml::CompositionChildRole zzc = c->childRole();
		string aname(MakeRoleName(zzp));
		
		output << "\t\t\tUml::InitCompositionParentRole(" << ((Uml::Class)zzc.target()).name() << "::meta_" << aname << 
		", comp, \""<< aname << "\", " << (zzp.isNavigable() ? "true, " : "false, ") <<
		((Uml::Class)zzp.target()).name() << "::meta);" << endl;
						
		string cname(MakeRoleName(zzc));
		output << "\t\t\tUml::InitCompositionChildRole(" << ((Uml::Class)zzp.target()).name() << "::meta_" << cname << 
			", comp, \""<< MakeShortRoleName(zzc) << "\", " << (zzc.isNavigable() ? "true, " : "false, ") <<
			zzc.min() << ", " << zzc.max() << ", " << ((Uml::Class)zzc.target()).name() << "::meta);" << endl;
	
		output << "\t\t}" << endl;
	}
};

void GenerateMetaCPPInitCrossAssociations(const Uml::Diagram & diagram, const Uml::Diagram & cross_dgr, ostream & output)
{
	set<Uml::Class> classes = diagram.classes();
	set<Uml::Class>::iterator c;
	//cross-association role equiv. generation
	if (cross_dgr &&(cross_dgr != diagram))
	{
		//		M2::meta_src = CL::M2::meta_src;		
		for(c = classes.begin(); c != classes.end(); c++ )	
		{
			Uml::Class cl = *c;
			Uml::Class cross_cl;
			if (cross_dgr)
			{
				//--//cross_cl = Uml::classByName(cross_dgr, cl.name());
				cross_cl = Uml::classByName(cross_dgr, (string(cl.name()) + string(Udm::cross_delimiter) + string(diagram.name())) );
			}
			if (cross_cl)
			{
				set<Uml::AssociationRole> ar = cross_cl.associationRoles();
				set<Uml::AssociationRole>::iterator ar_i = ar.begin();
				while (ar_i != ar.end())
				{
					output << "\t\t"  <<cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
						" = " << cross_dgr.name()<< "::" << cross_cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name()<< ";" <<endl;
					Uml::AssociationRole zz	= Uml::theOther(*ar_i);
					string aname(MakeRoleName(zz));
					Uml::Association ass = ar_i->parent();
					Uml::Class a_class = ass.assocClass();
						
					if(a_class) 
					{
						Uml::Class cl2 = zz.target();
						//output << "\t\t" << a_class.from() <<"::" << a_class.name() << "::meta_" << aname << "_end_ = " <<
						//					cl2.from() << "::" << cl2.name() << "::meta_" << MakeRoleName(*ar_i) << "_rev = " << 
						//					cl.name() << "::meta_" << aname << ";" << endl;

						output << "\t\t" << cross_cl.name() << "::meta_"<<Uml::theOther((*ar_i)).name() << 
						"_rev = " << cross_dgr.name()<< "::" << cl2.name() << "::meta_"<<ar_i->name()<< ";" <<endl;

					}
					ar_i++;
				}
			}
		}
	}
};
void GenerateMetaCPPInitInheritance(const Uml::Diagram &diagram, ostream & output)
{
	//Uml::AddInheritance(A::meta, C::meta);
	
	set<Uml::Class> classes = diagram.classes();
	set<Uml::Class>::iterator c;
	
	for(c = classes.begin(); c != classes.end(); c++ ) 
	{
		Uml::Class cl = *c;
		set<Uml::Class> st = cl.subTypes();
		for( set<Uml::Class>::iterator i = st.begin(); i != st.end(); i++) 
		{
			output << "\t\tUml::AddInheritance(" << cl.name() << "::meta, " << (*i).name() << "::meta);" << endl;
		}
	}			
};
void GenerateMetaCPPInitialize(const Uml::Diagram & diagram, const Uml::Diagram & cross_dgr, ostream & output, const string& macro, const string& hname)
{
		//Generate the Initialize() function

	output << "\t" << macro << " void Initialize()" << endl << "\t{" << endl;
	output << "\t\tstatic bool first = true;" << endl;
	output << "\t\tif(!first) return;" << endl;
	output << "\t\tfirst = false;" << endl;
	output << "\t\tUml::Initialize();" << endl << endl;

	if (cross_dgr && (diagram != cross_dgr))
	output << "\t\t" << cross_dgr.name() << "::Initialize();" << endl << endl;


	output << "\t\tASSERT( umldiagram == Udm::null );" << endl;

	//Creating part
	output << "\t\tumldiagram = Uml::CreateDiagram();" << endl << endl;
	GenerateMetaCPPCreates(diagram, output);

	//Initialization part

	//Uml::InitDiagram(diagram, "Sample");
	output << "\t\tUml::InitDiagram(umldiagram, \"" << hname << "\", \"" << diagram.version() << "\");" << endl << endl;

	//here: build the meta objects
	GenerateMetaCPPInitMetaObjects(diagram, cross_dgr, output);

	//build the association and composition relationships
	GenerateMetaCPPInitAssociations(diagram, output);
	GenerateMetaCPPInitCompositions(diagram, output);

	//build the cross association objects
	GenerateMetaCPPInitCrossAssociations(diagram, cross_dgr, output);

	//build the inheritence relationship
	GenerateMetaCPPInitInheritance(diagram, output);
	
	output << "\t\t" << endl << "\t}" << endl;
};

void GenerateCPP(const Uml::Diagram &diagram,  ostream &output, string fname, const Uml::Diagram& cross_dgr = NULL, const string& macro="") 
{
		string hname = diagram.name();
		string::iterator i;
		for(i = hname.begin(); i != hname.end(); i++) {
			if(!isalnum(*i)) *i = '_';
		}

		output << "// cpp source file " << fname << ".cpp generated from diagram " << (string)diagram.name() << endl;
		output << "// generated on " << GetTime().c_str() << endl << endl;
		output << "#include \""<< fname << ".h\"" <<endl;
		output << "#include \"UmlExt.h\"" <<endl << endl; 
//		output << "#include \"UdmStatic.h\"" << endl << endl; 



		if (cross_dgr && (diagram != cross_dgr))
		{
			output << "// cross-package metainformation header file" << endl;
			output << "#include \"" << cross_dgr.name() << ".h\"" << endl << endl; 
		}

		//begin of namespace
		output << "namespace " << hname << " {" << endl << endl;

		//declarations inside the namespace
		output << "\tUml::Diagram umldiagram;" << endl << endl;
		GenerateCPPDeclareMetaClasses(diagram, output);
		GenerateCPPDeclareAttributes(diagram, output);
		GenerateCPPDeclareAssociationRoles(diagram, cross_dgr, output);
		GenerateCPPDeclareCompositionRoles(diagram, output);
		GenerateCPPDeclareConstraints(diagram, output);
	
		//initialize function
		GenerateMetaCPPInitialize(diagram, cross_dgr, output, macro, hname);
		
		//Udm::UdmDiagram declaration
		output << "\t" << macro << " Udm::UdmDiagram diagram = { &umldiagram, Initialize };" << endl;
		//static registrator class
		GenerateCPPStaticClass(diagram, output);

		//end of namespace
		output << "}" << endl <<"// END " << hname << ".cpp" <<endl;
}

