#include "CrossLinkTester.h"
#include "CLPackage.h"
#include "CL.h"
#include "Package1.h"
#include "Package2.h"

CPPUNIT_TEST_SUITE_REGISTRATION( UdmTests::CrossLinkTester );

void UdmTests::CrossLinkTester::test()
{
	try {
		Udm::UdmProject prj;
		//specifying instance data networks which will be present in the project
		Udm::DataNetworkSpecifier dnsP1("Package1_inst.xml", "Package1", Package1::ClassA::meta);
		Udm::DataNetworkSpecifier dnsP2("Package2_inst.xml", "Package2", Package2::Class1::meta);
		vector<Udm::DataNetworkSpecifier> dnsvec;
		dnsvec.push_back(dnsP1);
		dnsvec.push_back(dnsP2);
		//creating a Udm Project
		prj.CreateNew("test.udm", dnsvec, CL::diagram, Udm::CHANGES_LOST_DEFAULT);
		//obtaining handles to the datanetworks which were created
		Udm::DataNetwork& p1 = prj.GetDataNetwork("Package1_inst.xml");
		Udm::DataNetwork& p2 = prj.GetDataNetwork("Package2_inst.xml");
		//obtaining handles to the root objects (this is normal - existing - Udm operation)
		Package1::ClassA rootP1 = Package1::ClassA::Cast( p1.GetRootObject());
		Package2::Class1 rootP2 = Package2::Class1::Cast( p2.GetRootObject());
		// create Class2
		Package2::Class2 class2= Package2::Class2::Create( rootP2);
		// create cross links
		rootP1.dst1()+= rootP2;
		rootP1.dst1()+= class2;
		rootP1.dst2()= class2;
		// verification
		std::set< Package2::Class1> dst1s= rootP1.dst1();
		CPPUNIT_ASSERT( dst1s.size() == 2);
		CPPUNIT_ASSERT( dst1s.find( rootP2) != dst1s.end());
		CPPUNIT_ASSERT( dst1s.find( class2) != dst1s.end());
		const Package2::Class2& dst2= rootP1.dst2();
		CPPUNIT_ASSERT( dst2 == class2);
	} catch( udm_exception &e)
	{
		std::cout << e.what() << std::endl;
		CPPUNIT_ASSERT( false);
	}
}


